@@ -18,10 +18,6 @@
 
 package org.apache.flink.table.store.codegen;
 
-import org.apache.flink.table.runtime.generated.GeneratedClass;
-import org.apache.flink.table.runtime.generated.GeneratedNormalizedKeyComputer;
-import org.apache.flink.table.runtime.generated.GeneratedProjection;
-import org.apache.flink.table.runtime.generated.GeneratedRecordComparator;
 import org.apache.flink.table.types.logical.LogicalType;
 import org.apache.flink.table.types.logical.RowType;
 
@@ -30,24 +26,26 @@ import java.util.List;
 /** {@link GeneratedClass} generator. */
 public interface CodeGenerator {
 
-    GeneratedProjection generateProjection(String name, RowType inputType, int[] inputMapping);
+    GeneratedClass<Projection> generateProjection(
+            String name, RowType inputType, int[] inputMapping);
 
     /**
-     * Generate a {@link GeneratedNormalizedKeyComputer}.
+     * Generate a {@link NormalizedKeyComputer}.
      *
      * @param fieldTypes Both the input row field types and the sort key field types. Records are
      *     compared by the first field, then the second field, then the third field and so on. All
      *     fields are compared in ascending order.
      */
-    GeneratedNormalizedKeyComputer generateNormalizedKeyComputer(
+    GeneratedClass<NormalizedKeyComputer> generateNormalizedKeyComputer(
             List<LogicalType> fieldTypes, String name);
 
     /**
-     * Generate a {@link GeneratedRecordComparator}.
+     * Generate a {@link RecordComparator}.
      *
      * @param fieldTypes Both the input row field types and the sort key field types. Records are *
      *     compared by the first field, then the second field, then the third field and so on. All *
      *     fields are compared in ascending order.
      */
-    GeneratedRecordComparator generateRecordComparator(List<LogicalType> fieldTypes, String name);
+    GeneratedClass<RecordComparator> generateRecordComparator(
+            List<LogicalType> fieldTypes, String name);
 }
