@@ -19,10 +19,10 @@
 package org.apache.flink.table.store.connector;
 
 import org.apache.flink.annotation.VisibleForTesting;
-import org.apache.flink.core.fs.Path;
 import org.apache.flink.table.catalog.AbstractCatalog;
 import org.apache.flink.table.catalog.CatalogBaseTable;
 import org.apache.flink.table.catalog.CatalogDatabase;
+import org.apache.flink.table.catalog.CatalogDatabaseImpl;
 import org.apache.flink.table.catalog.CatalogFunction;
 import org.apache.flink.table.catalog.CatalogPartition;
 import org.apache.flink.table.catalog.CatalogPartitionSpec;
@@ -42,8 +42,9 @@ import org.apache.flink.table.expressions.Expression;
 import org.apache.flink.table.factories.Factory;
 import org.apache.flink.table.store.file.catalog.Catalog;
 import org.apache.flink.table.store.file.schema.SchemaChange;
-import org.apache.flink.table.store.file.schema.TableSchema;
 import org.apache.flink.table.store.file.schema.UpdateSchema;
+import org.apache.flink.table.store.table.FileStoreTable;
+import org.apache.flink.table.store.table.Table;
 
 import java.util.ArrayList;
 import java.util.Collections;
@@ -89,6 +90,15 @@ public class FlinkCatalog extends AbstractCatalog {
         return catalog.databaseExists(databaseName);
     }
 
+    @Override
+    public CatalogDatabase getDatabase(String databaseName)
+            throws CatalogException, DatabaseNotExistException {
+        if (databaseExists(databaseName)) {
+            return new CatalogDatabaseImpl(Collections.emptyMap(), null);
+        }
+        throw new DatabaseNotExistException(getName(), databaseName);
+    }
+
     @Override
     public void createDatabase(String name, CatalogDatabase database, boolean ignoreIfExists)
             throws DatabaseAlreadyExistException, CatalogException {
@@ -137,17 +147,24 @@ public class FlinkCatalog extends AbstractCatalog {
     @Override
     public CatalogTable getTable(ObjectPath tablePath)
             throws TableNotExistException, CatalogException {
-        TableSchema schema;
+        Table table;
         try {
-            schema = catalog.getTableSchema(tablePath);
+            table = catalog.getTable(tablePath);
         } catch (Catalog.TableNotExistException e) {
             throw new TableNotExistException(getName(), e.tablePath());
         }
 
-        CatalogTable table = schema.toUpdateSchema().toCatalogTable();
-        // add path to source and sink
-        table.getOptions().put(PATH.key(), catalog.getTableLocation(tablePath).toString());
-        return table;
+        if (table instanceof FileStoreTable) {
+            CatalogTable catalogTable =
+                    ((FileStoreTable) table).schema().toUpdateSchema().toCatalogTable();
+            // add path to source and sink
+            catalogTable
+                    .getOptions()
+                    .put(PATH.key(), catalog.getTableLocation(tablePath).toString());
+            return catalogTable;
+        } else {
+            return new SystemCatalogTable(table);
+        }
     }
 
     @Override
@@ -185,10 +202,6 @@ public class FlinkCatalog extends AbstractCatalog {
         // remove table path
         String specific = options.remove(PATH.key());
         if (specific != null) {
-            if (!catalog.getTableLocation(tablePath).equals(new Path(specific))) {
-                throw new IllegalArgumentException(
-                        "Illegal table path in table options: " + specific);
-            }
             catalogTable = catalogTable.copy(options);
         }
 
@@ -291,11 +304,6 @@ public class FlinkCatalog extends AbstractCatalog {
 
     // --------------------- unsupported methods ----------------------------
 
-    @Override
-    public CatalogDatabase getDatabase(String databaseName) throws CatalogException {
-        throw new UnsupportedOperationException();
-    }
-
     @Override
     public final void alterDatabase(
             String name, CatalogDatabase newDatabase, boolean ignoreIfNotExists)
