@@ -25,6 +25,8 @@ import org.apache.flink.connector.file.src.util.RecordAndPosition;
 import org.apache.flink.table.data.RowData;
 import org.apache.flink.table.store.table.source.TableRead;
 
+import javax.annotation.Nullable;
+
 import java.util.Map;
 
 /** A {@link SourceReader} that read records from {@link FileStoreSourceSplit}. */
@@ -35,9 +37,10 @@ public final class FileStoreSourceReader
                 FileStoreSourceSplit,
                 FileStoreSourceSplitState> {
 
-    public FileStoreSourceReader(SourceReaderContext readerContext, TableRead tableRead) {
+    public FileStoreSourceReader(
+            SourceReaderContext readerContext, TableRead tableRead, @Nullable Long limit) {
         super(
-                () -> new FileStoreSourceSplitReader(tableRead),
+                () -> new FileStoreSourceSplitReader(tableRead, limit),
                 (element, output, splitState) -> {
                     output.collect(element.getRecord());
                     splitState.setPosition(element);
