@@ -36,7 +36,7 @@ public class FileStoreSourceSplitGenerator {
     private final char[] currentId = "0000000000".toCharArray();
 
     public List<FileStoreSourceSplit> createSplits(TableScan.Plan plan) {
-        return plan.splits.stream()
+        return plan.splits().stream()
                 .map(s -> new FileStoreSourceSplit(getNextId(), s))
                 .collect(Collectors.toList());
     }
