@@ -24,6 +24,7 @@ import org.apache.flink.table.api.DataTypes;
 import org.apache.flink.table.api.Schema;
 import org.apache.flink.table.catalog.Catalog;
 import org.apache.flink.table.catalog.CatalogBaseTable;
+import org.apache.flink.table.catalog.CatalogDatabase;
 import org.apache.flink.table.catalog.CatalogDatabaseImpl;
 import org.apache.flink.table.catalog.CatalogTable;
 import org.apache.flink.table.catalog.Column;
@@ -74,7 +75,9 @@ public class FlinkCatalogTest {
         String path = TEMPORARY_FOLDER.newFolder().toURI().toString();
         Configuration conf = new Configuration();
         conf.setString("warehouse", path);
-        catalog = FlinkCatalogFactory.createCatalog("test-catalog", conf);
+        catalog =
+                FlinkCatalogFactory.createCatalog(
+                        "test-catalog", conf, FlinkCatalogTest.class.getClassLoader());
     }
 
     private ResolvedSchema createSchema() {
@@ -323,6 +326,17 @@ public class FlinkCatalogTest {
         Assert.assertTrue(catalog.databaseExists("db1"));
     }
 
+    @Test
+    public void testGetDatabase() throws Exception {
+        catalog.createDatabase(path1.getDatabaseName(), null, false);
+        CatalogDatabase database = catalog.getDatabase(path1.getDatabaseName());
+        assertThat(database.getProperties()).isEmpty();
+        assertThat(database.getDescription()).isEmpty();
+        assertThatThrownBy(() -> catalog.getDatabase(nonExistDbPath.getDatabaseName()))
+                .isInstanceOf(DatabaseNotExistException.class)
+                .hasMessageContaining("Database non does not exist in Catalog test-catalog.");
+    }
+
     @Test
     public void testDropDb_DatabaseNotExist_Ignore() throws Exception {
         catalog.dropDatabase("db1", true, false);
