@@ -18,13 +18,14 @@
 
 package org.apache.flink.table.store.connector.source;
 
-import org.apache.flink.table.store.file.data.DataFileMeta;
+import org.apache.flink.table.store.file.io.DataFileMeta;
 import org.apache.flink.table.store.file.manifest.FileKind;
 import org.apache.flink.table.store.file.manifest.ManifestEntry;
 import org.apache.flink.table.store.file.operation.FileStoreScan;
 import org.apache.flink.table.store.file.stats.StatsTestUtils;
-import org.apache.flink.table.store.table.source.Split;
-import org.apache.flink.table.store.table.source.TableScan;
+import org.apache.flink.table.store.table.source.AbstractDataTableScan;
+import org.apache.flink.table.store.table.source.DataSplit;
+import org.apache.flink.table.store.table.source.DataTableScan;
 
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
@@ -75,18 +76,25 @@ public class FileStoreSourceSplitGeneratorTest {
                                 makeEntry(6, 1, "f14"));
                     }
                 };
-        List<Split> scanSplits =
-                TableScan.generateSplits(
-                        false, Collections::singletonList, plan.groupByPartFiles());
-        TableScan.Plan tableScanPlan = new TableScan.Plan(1L, scanSplits);
+        List<DataSplit> scanSplits =
+                AbstractDataTableScan.generateSplits(
+                        1L,
+                        false,
+                        Collections::singletonList,
+                        plan.groupByPartFiles(plan.files(FileKind.ADD)));
+        DataTableScan.DataFilePlan tableScanPlan = new DataTableScan.DataFilePlan(1L, scanSplits);
 
         List<FileStoreSourceSplit> splits =
                 new FileStoreSourceSplitGenerator().createSplits(tableScanPlan);
         assertThat(splits.size()).isEqualTo(12);
         splits.sort(
                 Comparator.comparingInt(
-                                o -> ((FileStoreSourceSplit) o).split().partition().getInt(0))
-                        .thenComparing(o -> ((FileStoreSourceSplit) o).split().bucket()));
+                                o ->
+                                        ((DataSplit) ((FileStoreSourceSplit) o).split())
+                                                .partition()
+                                                .getInt(0))
+                        .thenComparing(
+                                o -> ((DataSplit) ((FileStoreSourceSplit) o).split()).bucket()));
         assertSplit(splits.get(0), "0000000007", 1, 0, Arrays.asList("f0", "f1"));
         assertSplit(splits.get(1), "0000000008", 1, 1, Collections.singletonList("f2"));
         assertSplit(splits.get(2), "0000000003", 2, 0, Arrays.asList("f3", "f4", "f5"));
@@ -104,12 +112,13 @@ public class FileStoreSourceSplitGeneratorTest {
     private void assertSplit(
             FileStoreSourceSplit split, String splitId, int part, int bucket, List<String> files) {
         assertThat(split.splitId()).isEqualTo(splitId);
-        assertThat(split.split().partition().getInt(0)).isEqualTo(part);
-        assertThat(split.split().bucket()).isEqualTo(bucket);
+        assertThat(((DataSplit) split.split()).partition().getInt(0)).isEqualTo(part);
+        assertThat(((DataSplit) split.split()).bucket()).isEqualTo(bucket);
         assertThat(
-                        split.split().files().stream()
-                                .map(DataFileMeta::fileName)
-                                .collect(Collectors.toList()))
+                        ((DataSplit) split.split())
+                                .files().stream()
+                                        .map(DataFileMeta::fileName)
+                                        .collect(Collectors.toList()))
                 .isEqualTo(files);
     }
 
