@@ -23,15 +23,22 @@ import org.apache.flink.connector.base.source.reader.RecordsWithSplitIds;
 import org.apache.flink.connector.base.source.reader.splitreader.SplitsAddition;
 import org.apache.flink.connector.base.source.reader.splitreader.SplitsChange;
 import org.apache.flink.connector.file.src.util.RecordAndPosition;
+import org.apache.flink.core.fs.Path;
 import org.apache.flink.table.data.GenericRowData;
 import org.apache.flink.table.data.RowData;
 import org.apache.flink.table.store.file.KeyValue;
-import org.apache.flink.table.store.file.data.DataFileMeta;
-import org.apache.flink.table.store.file.writer.RecordWriter;
+import org.apache.flink.table.store.file.io.DataFileMeta;
+import org.apache.flink.table.store.file.schema.SchemaManager;
+import org.apache.flink.table.store.file.schema.UpdateSchema;
+import org.apache.flink.table.store.file.utils.RecordWriter;
+import org.apache.flink.table.types.logical.BigIntType;
+import org.apache.flink.table.types.logical.IntType;
+import org.apache.flink.table.types.logical.RowType;
 import org.apache.flink.types.RowKind;
 
 import org.junit.jupiter.api.AfterAll;
 import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
 
@@ -67,6 +74,22 @@ public class FileStoreSourceSplitReaderTest {
         service = null;
     }
 
+    @BeforeEach
+    public void beforeEach() throws Exception {
+        SchemaManager schemaManager = new SchemaManager(new Path(tempDir.toUri()));
+        schemaManager.commitNewVersion(
+                new UpdateSchema(
+                        new RowType(
+                                Arrays.asList(
+                                        new RowType.RowField("k", new BigIntType()),
+                                        new RowType.RowField("v", new BigIntType()),
+                                        new RowType.RowField("default", new IntType()))),
+                        Collections.singletonList("default"),
+                        Arrays.asList("k", "default"),
+                        Collections.emptyMap(),
+                        null));
+    }
+
     @Test
     public void testPrimaryKey() throws Exception {
         innerTestOnce(false, 0);
@@ -91,7 +114,8 @@ public class FileStoreSourceSplitReaderTest {
         TestChangelogDataReadWrite rw = new TestChangelogDataReadWrite(tempDir.toString(), service);
         FileStoreSourceSplitReader reader =
                 new FileStoreSourceSplitReader(
-                        valueCountMode ? rw.createReadWithValueCount() : rw.createReadWithKey());
+                        valueCountMode ? rw.createReadWithValueCount() : rw.createReadWithKey(),
+                        null);
 
         List<Tuple2<Long, Long>> input = kvs();
         List<DataFileMeta> files = rw.writeFiles(row(1), 0, input);
@@ -132,7 +156,8 @@ public class FileStoreSourceSplitReaderTest {
     @Test
     public void testPrimaryKeyWithDelete() throws Exception {
         TestChangelogDataReadWrite rw = new TestChangelogDataReadWrite(tempDir.toString(), service);
-        FileStoreSourceSplitReader reader = new FileStoreSourceSplitReader(rw.createReadWithKey());
+        FileStoreSourceSplitReader reader =
+                new FileStoreSourceSplitReader(rw.createReadWithKey(), null);
 
         List<Tuple2<Long, Long>> input = kvs();
         RecordWriter<KeyValue> writer = rw.createMergeTreeWriter(row(1), 0);
@@ -147,7 +172,7 @@ public class FileStoreSourceSplitReaderTest {
         writer.write(
                 new KeyValue()
                         .replace(GenericRowData.of(222L), RowKind.DELETE, GenericRowData.of(333L)));
-        List<DataFileMeta> files = writer.prepareCommit(true).newFiles();
+        List<DataFileMeta> files = writer.prepareCommit(true).newFilesIncrement().newFiles();
         writer.close();
 
         assignSplit(reader, newSourceSplit("id1", row(1), 0, files, true));
@@ -171,7 +196,8 @@ public class FileStoreSourceSplitReaderTest {
     @Test
     public void testMultipleBatchInSplit() throws Exception {
         TestChangelogDataReadWrite rw = new TestChangelogDataReadWrite(tempDir.toString(), service);
-        FileStoreSourceSplitReader reader = new FileStoreSourceSplitReader(rw.createReadWithKey());
+        FileStoreSourceSplitReader reader =
+                new FileStoreSourceSplitReader(rw.createReadWithKey(), null);
 
         List<Tuple2<Long, Long>> input1 = kvs();
         List<DataFileMeta> files = rw.writeFiles(row(1), 0, input1);
@@ -207,7 +233,8 @@ public class FileStoreSourceSplitReaderTest {
     @Test
     public void testRestore() throws Exception {
         TestChangelogDataReadWrite rw = new TestChangelogDataReadWrite(tempDir.toString(), service);
-        FileStoreSourceSplitReader reader = new FileStoreSourceSplitReader(rw.createReadWithKey());
+        FileStoreSourceSplitReader reader =
+                new FileStoreSourceSplitReader(rw.createReadWithKey(), null);
 
         List<Tuple2<Long, Long>> input = kvs();
         List<DataFileMeta> files = rw.writeFiles(row(1), 0, input);
@@ -233,7 +260,8 @@ public class FileStoreSourceSplitReaderTest {
     @Test
     public void testRestoreMultipleBatchInSplit() throws Exception {
         TestChangelogDataReadWrite rw = new TestChangelogDataReadWrite(tempDir.toString(), service);
-        FileStoreSourceSplitReader reader = new FileStoreSourceSplitReader(rw.createReadWithKey());
+        FileStoreSourceSplitReader reader =
+                new FileStoreSourceSplitReader(rw.createReadWithKey(), null);
 
         List<Tuple2<Long, Long>> input1 = kvs();
         List<DataFileMeta> files = rw.writeFiles(row(1), 0, input1);
@@ -264,7 +292,8 @@ public class FileStoreSourceSplitReaderTest {
     @Test
     public void testMultipleSplits() throws Exception {
         TestChangelogDataReadWrite rw = new TestChangelogDataReadWrite(tempDir.toString(), service);
-        FileStoreSourceSplitReader reader = new FileStoreSourceSplitReader(rw.createReadWithKey());
+        FileStoreSourceSplitReader reader =
+                new FileStoreSourceSplitReader(rw.createReadWithKey(), null);
 
         List<Tuple2<Long, Long>> input1 = kvs();
         List<DataFileMeta> files1 = rw.writeFiles(row(1), 0, input1);
@@ -302,11 +331,39 @@ public class FileStoreSourceSplitReaderTest {
     @Test
     public void testNoSplit() throws Exception {
         TestChangelogDataReadWrite rw = new TestChangelogDataReadWrite(tempDir.toString(), service);
-        FileStoreSourceSplitReader reader = new FileStoreSourceSplitReader(rw.createReadWithKey());
+        FileStoreSourceSplitReader reader =
+                new FileStoreSourceSplitReader(rw.createReadWithKey(), null);
         assertThatThrownBy(reader::fetch).hasMessageContaining("no split remaining");
         reader.close();
     }
 
+    @Test
+    public void testLimit() throws Exception {
+        TestChangelogDataReadWrite rw = new TestChangelogDataReadWrite(tempDir.toString(), service);
+        FileStoreSourceSplitReader reader =
+                new FileStoreSourceSplitReader(rw.createReadWithKey(), 2L);
+
+        List<Tuple2<Long, Long>> input = kvs();
+        List<DataFileMeta> files = rw.writeFiles(row(1), 0, input);
+
+        assignSplit(reader, newSourceSplit("id1", row(1), 0, files, 0));
+
+        RecordsWithSplitIds<RecordAndPosition<RowData>> records = reader.fetch();
+
+        List<Tuple2<RowKind, Long>> expected =
+                input.stream()
+                        .map(t -> new Tuple2<>(RowKind.INSERT, t.f1))
+                        .collect(Collectors.toList());
+
+        List<Tuple2<RowKind, Long>> result = readRecords(records, "id1", 0);
+        assertThat(result).isEqualTo(expected.subList(0, 2));
+
+        records = reader.fetch();
+        assertRecords(records, null, "id1", 0, Collections.emptyList());
+
+        reader.close();
+    }
+
     private void assertRecords(
             RecordsWithSplitIds<RecordAndPosition<RowData>> records,
             String finishedSplit,
