@@ -25,23 +25,37 @@ import org.apache.flink.configuration.ConfigOptions;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.DescribedEnum;
 import org.apache.flink.configuration.MemorySize;
+import org.apache.flink.configuration.ReadableConfig;
 import org.apache.flink.configuration.description.Description;
 import org.apache.flink.configuration.description.InlineElement;
 import org.apache.flink.core.fs.Path;
+import org.apache.flink.table.api.TableException;
 import org.apache.flink.table.store.file.WriteMode;
+import org.apache.flink.table.store.file.schema.TableSchema;
 import org.apache.flink.table.store.format.FileFormat;
 import org.apache.flink.util.Preconditions;
 
 import java.io.Serializable;
+import java.lang.annotation.ElementType;
+import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
+import java.lang.annotation.Target;
 import java.lang.reflect.Field;
 import java.time.Duration;
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Optional;
+import java.util.Set;
 
 import static org.apache.flink.configuration.ConfigOptions.key;
 import static org.apache.flink.configuration.description.TextElement.text;
+import static org.apache.flink.table.store.file.WriteMode.APPEND_ONLY;
+import static org.apache.flink.table.store.file.schema.TableSchema.KEY_FIELD_PREFIX;
+import static org.apache.flink.table.store.file.schema.TableSchema.SYSTEM_FIELD_NAMES;
+import static org.apache.flink.util.Preconditions.checkState;
 
 /** Core options for table store. */
 public class CoreOptions implements Serializable {
@@ -52,6 +66,7 @@ public class CoreOptions implements Serializable {
                     .defaultValue(1)
                     .withDescription("Bucket number for file store.");
 
+    @Immutable
     public static final ConfigOption<String> BUCKET_KEY =
             ConfigOptions.key("bucket-key")
                     .stringType()
@@ -135,18 +150,35 @@ public class CoreOptions implements Serializable {
                     .defaultValue(Duration.ofSeconds(1))
                     .withDescription("The discovery interval of continuous reading.");
 
+    @Immutable
     public static final ConfigOption<MergeEngine> MERGE_ENGINE =
             ConfigOptions.key("merge-engine")
                     .enumType(MergeEngine.class)
                     .defaultValue(MergeEngine.DEDUPLICATE)
                     .withDescription("Specify the merge engine for table with primary key.");
 
+    public static final ConfigOption<Boolean> PARTIAL_UPDATE_IGNORE_DELETE =
+            ConfigOptions.key("partial-update.ignore-delete")
+                    .booleanType()
+                    .defaultValue(false)
+                    .withDescription("Whether to ignore delete records in partial-update mode.");
+
+    @Immutable
     public static final ConfigOption<WriteMode> WRITE_MODE =
             ConfigOptions.key("write-mode")
                     .enumType(WriteMode.class)
                     .defaultValue(WriteMode.CHANGE_LOG)
                     .withDescription("Specify the write mode for table.");
 
+    public static final ConfigOption<Boolean> WRITE_ONLY =
+            ConfigOptions.key("write-only")
+                    .booleanType()
+                    .defaultValue(false)
+                    .withDeprecatedKeys("write.compaction-skip")
+                    .withDescription(
+                            "If set to true, compactions and snapshot expiration will be skipped. "
+                                    + "This option is used along with dedicated compact jobs.");
+
     public static final ConfigOption<MemorySize> SOURCE_SPLIT_TARGET_SIZE =
             ConfigOptions.key("source.split.target-size")
                     .memoryType()
@@ -168,6 +200,22 @@ public class CoreOptions implements Serializable {
                     .withDescription(
                             "Amount of data to build up in memory before converting to a sorted on-disk file.");
 
+    public static final ConfigOption<Boolean> WRITE_BUFFER_SPILLABLE =
+            ConfigOptions.key("write-buffer-spillable")
+                    .booleanType()
+                    .noDefaultValue()
+                    .withDescription(
+                            "Whether the write buffer can be spillable. Enabled by default when using object storage.");
+
+    public static final ConfigOption<Integer> LOCAL_SORT_MAX_NUM_FILE_HANDLES =
+            ConfigOptions.key("local-sort.max-num-file-handles")
+                    .intType()
+                    .defaultValue(128)
+                    .withDescription(
+                            "The maximal fan-in for external merge sort. It limits the number of file handles. "
+                                    + "If it is too small, may cause intermediate merging. But if it is too large, "
+                                    + "it will cause too many files opened at the same time, consume memory and lead to random reading.");
+
     public static final ConfigOption<MemorySize> PAGE_SIZE =
             ConfigOptions.key("page-size")
                     .memoryType()
@@ -263,6 +311,18 @@ public class CoreOptions implements Serializable {
                                     + "This changelog file keeps the details of data changes, "
                                     + "it can be read directly during stream reads.");
 
+    public static final ConfigOption<Duration> CHANGELOG_PRODUCER_FULL_COMPACTION_TRIGGER_INTERVAL =
+            ConfigOptions.key("changelog-producer.compaction-interval")
+                    .durationType()
+                    .defaultValue(Duration.ofMinutes(30))
+                    .withDescription(
+                            "When "
+                                    + CHANGELOG_PRODUCER.key()
+                                    + " is set to "
+                                    + ChangelogProducer.FULL_COMPACTION.name()
+                                    + ", full compaction will be constantly triggered after this interval.");
+
+    @Immutable
     public static final ConfigOption<String> SEQUENCE_FIELD =
             ConfigOptions.key("sequence.field")
                     .stringType()
@@ -271,18 +331,27 @@ public class CoreOptions implements Serializable {
                             "The field that generates the sequence number for primary key table,"
                                     + " the sequence number determines which data is the most recent.");
 
-    public static final ConfigOption<LogStartupMode> LOG_SCAN =
-            ConfigOptions.key("log.scan")
-                    .enumType(LogStartupMode.class)
-                    .defaultValue(LogStartupMode.FULL)
-                    .withDescription("Specify the startup mode for log consumer.");
+    public static final ConfigOption<StartupMode> SCAN_MODE =
+            ConfigOptions.key("scan.mode")
+                    .enumType(StartupMode.class)
+                    .defaultValue(StartupMode.DEFAULT)
+                    .withDeprecatedKeys("log.scan")
+                    .withDescription("Specify the scanning behavior of the source.");
 
-    public static final ConfigOption<Long> LOG_SCAN_TIMESTAMP_MILLS =
-            ConfigOptions.key("log.scan.timestamp-millis")
+    public static final ConfigOption<Long> SCAN_TIMESTAMP_MILLIS =
+            ConfigOptions.key("scan.timestamp-millis")
+                    .longType()
+                    .noDefaultValue()
+                    .withDeprecatedKeys("log.scan.timestamp-millis")
+                    .withDescription(
+                            "Optional timestamp used in case of \"from-timestamp\" scan mode.");
+
+    public static final ConfigOption<Long> SCAN_SNAPSHOT_ID =
+            ConfigOptions.key("scan.snapshot-id")
                     .longType()
                     .noDefaultValue()
                     .withDescription(
-                            "Optional timestamp used in case of \"from-timestamp\" scan mode");
+                            "Optional snapshot id used in case of \"from-snapshot\" scan mode");
 
     public static final ConfigOption<Duration> LOG_RETENTION =
             ConfigOptions.key("log.retention")
@@ -325,6 +394,13 @@ public class CoreOptions implements Serializable {
                     .defaultValue("debezium-json")
                     .withDescription("Specify the message format of log system.");
 
+    public static final ConfigOption<Boolean> AUTO_CREATE =
+            ConfigOptions.key("auto-create")
+                    .booleanType()
+                    .defaultValue(false)
+                    .withDescription(
+                            "Whether to create underlying storage when reading and writing the table.");
+
     private final Configuration options;
 
     public CoreOptions(Map<String, String> options) {
@@ -333,15 +409,10 @@ public class CoreOptions implements Serializable {
 
     public CoreOptions(Configuration options) {
         this.options = options;
-        // TODO validate all keys
-        Preconditions.checkArgument(
-                snapshotNumRetainMin() > 0,
-                SNAPSHOT_NUM_RETAINED_MIN.key() + " should be at least 1");
-        Preconditions.checkArgument(
-                snapshotNumRetainMin() <= snapshotNumRetainMax(),
-                SNAPSHOT_NUM_RETAINED_MIN.key()
-                        + " should not be larger than "
-                        + SNAPSHOT_NUM_RETAINED_MAX.key());
+    }
+
+    public Map<String, String> toMap() {
+        return options.toMap();
     }
 
     public int bucket() {
@@ -392,6 +463,10 @@ public class CoreOptions implements Serializable {
         return options.get(MANIFEST_MERGE_MIN_COUNT);
     }
 
+    public MergeEngine mergeEngine() {
+        return options.get(MERGE_ENGINE);
+    }
+
     public long splitTargetSize() {
         return options.get(SOURCE_SPLIT_TARGET_SIZE).getBytes();
     }
@@ -404,6 +479,18 @@ public class CoreOptions implements Serializable {
         return options.get(WRITE_BUFFER_SIZE).getBytes();
     }
 
+    public boolean writeBufferSpillable(boolean usingObjectStore) {
+        return options.getOptional(WRITE_BUFFER_SPILLABLE).orElse(usingObjectStore);
+    }
+
+    public Duration continuousDiscoveryInterval() {
+        return options.get(CONTINUOUS_DISCOVERY_INTERVAL);
+    }
+
+    public int localSortMaxNumFileHandles() {
+        return options.get(LOCAL_SORT_MAX_NUM_FILE_HANDLES);
+    }
+
     public int pageSize() {
         return (int) options.get(PAGE_SIZE).getBytes();
     }
@@ -448,11 +535,11 @@ public class CoreOptions implements Serializable {
         return options.get(COMPACTION_SIZE_RATIO);
     }
 
-    public int minFileNum() {
+    public int compactionMinFileNum() {
         return options.get(COMPACTION_MIN_FILE_NUM);
     }
 
-    public int maxFileNum() {
+    public int compactionMaxFileNum() {
         return options.get(COMPACTION_MAX_FILE_NUM);
     }
 
@@ -464,15 +551,58 @@ public class CoreOptions implements Serializable {
         return options.get(CHANGELOG_PRODUCER);
     }
 
+    public StartupMode startupMode() {
+        return startupMode(options);
+    }
+
+    public static StartupMode startupMode(ReadableConfig options) {
+        StartupMode mode = options.get(SCAN_MODE);
+        if (mode == StartupMode.DEFAULT) {
+            if (options.getOptional(SCAN_TIMESTAMP_MILLIS).isPresent()) {
+                return StartupMode.FROM_TIMESTAMP;
+            } else if (options.getOptional(SCAN_SNAPSHOT_ID).isPresent()) {
+                return StartupMode.FROM_SNAPSHOT;
+            } else {
+                return StartupMode.LATEST_FULL;
+            }
+        } else if (mode == StartupMode.FULL) {
+            return StartupMode.LATEST_FULL;
+        } else {
+            return mode;
+        }
+    }
+
+    public Long scanTimestampMills() {
+        return options.get(SCAN_TIMESTAMP_MILLIS);
+    }
+
+    public Long scanSnapshotId() {
+        return options.get(SCAN_SNAPSHOT_ID);
+    }
+
+    public Duration changelogProducerFullCompactionTriggerInterval() {
+        return options.get(CHANGELOG_PRODUCER_FULL_COMPACTION_TRIGGER_INTERVAL);
+    }
+
     public Optional<String> sequenceField() {
         return options.getOptional(SEQUENCE_FIELD);
     }
 
+    public WriteMode writeMode() {
+        return options.get(WRITE_MODE);
+    }
+
+    public boolean writeOnly() {
+        return options.get(WRITE_ONLY);
+    }
+
     /** Specifies the merge engine for table with primary key. */
     public enum MergeEngine implements DescribedEnum {
         DEDUPLICATE("deduplicate", "De-duplicate and keep the last row."),
 
-        PARTIAL_UPDATE("partial-update", "Partial update non-null fields.");
+        PARTIAL_UPDATE("partial-update", "Partial update non-null fields."),
+
+        AGGREGATE("aggregation", "Aggregate fields with same primary key.");
 
         private final String value;
         private final String description;
@@ -494,20 +624,54 @@ public class CoreOptions implements Serializable {
     }
 
     /** Specifies the startup mode for log consumer. */
-    public enum LogStartupMode implements DescribedEnum {
-        FULL(
-                "full",
-                "Perform a snapshot on the table upon first startup,"
-                        + " and continue to read the latest changes."),
+    public enum StartupMode implements DescribedEnum {
+        DEFAULT(
+                "default",
+                "Determines actual startup mode according to other table properties. "
+                        + "If \"scan.timestamp-millis\" is set the actual startup mode will be \"from-timestamp\", "
+                        + "and if \"scan.snapshot-id\" is set the actual startup mode will be \"from-snapshot\". "
+                        + "Otherwise the actual startup mode will be \"latest-full\"."),
 
-        LATEST("latest", "Start from the latest."),
+        LATEST_FULL(
+                "latest-full",
+                "For streaming sources, produces the latest snapshot on the table upon first startup, "
+                        + "and continue to read the latest changes. "
+                        + "For batch sources, just produce the latest snapshot but does not read new changes."),
 
-        FROM_TIMESTAMP("from-timestamp", "Start from user-supplied timestamp.");
+        FULL("full", "Deprecated. Same as \"latest-full\"."),
+
+        LATEST(
+                "latest",
+                "For streaming sources, continuously reads latest changes "
+                        + "without producing a snapshot at the beginning. "
+                        + "For batch sources, behaves the same as the \"latest-full\" startup mode."),
+
+        COMPACTED_FULL(
+                "compacted-full",
+                "For streaming sources, produces a snapshot after the latest compaction on the table "
+                        + "upon first startup, and continue to read the latest changes. "
+                        + "For batch sources, just produce a snapshot after the latest compaction "
+                        + "but does not read new changes."),
+
+        FROM_TIMESTAMP(
+                "from-timestamp",
+                "For streaming sources, continuously reads changes "
+                        + "starting from timestamp specified by \"scan.timestamp-millis\", "
+                        + "without producing a snapshot at the beginning. "
+                        + "For batch sources, produces a snapshot at timestamp specified by \"scan.timestamp-millis\" "
+                        + "but does not read new changes."),
+
+        FROM_SNAPSHOT(
+                "from-snapshot",
+                "For streaming sources, continuously reads changes "
+                        + "starting from snapshot specified by \"scan.snapshot-id\", "
+                        + "without producing a snapshot at the beginning. For batch sources, "
+                        + "produces a snapshot specified by \"scan.snapshot-id\" but does not read new changes.");
 
         private final String value;
         private final String description;
 
-        LogStartupMode(String value, String description) {
+        StartupMode(String value, String description) {
             this.value = value;
             this.description = description;
         }
@@ -555,7 +719,7 @@ public class CoreOptions implements Serializable {
 
     /** Specifies the log changelog mode for table. */
     public enum LogChangelogMode implements DescribedEnum {
-        AUTO("auto", "Upsert for table with primary key, all for table without primary key.."),
+        AUTO("auto", "Upsert for table with primary key, all for table without primary key."),
 
         ALL("all", "The log system stores all changes including UPDATE_BEFORE."),
 
@@ -590,7 +754,9 @@ public class CoreOptions implements Serializable {
 
         INPUT(
                 "input",
-                "Double write to a changelog file when flushing memory table, the changelog is from input.");
+                "Double write to a changelog file when flushing memory table, the changelog is from input."),
+
+        FULL_COMPACTION("full-compaction", "Generate changelog files with each full compaction.");
 
         private final String value;
         private final String description;
@@ -611,6 +777,104 @@ public class CoreOptions implements Serializable {
         }
     }
 
+    /**
+     * Set the default values of the {@link CoreOptions} via the given {@link Configuration}.
+     *
+     * @param options the options to set default values
+     */
+    public static void setDefaultValues(Configuration options) {
+        if (options.contains(SCAN_TIMESTAMP_MILLIS) && !options.contains(SCAN_MODE)) {
+            options.set(SCAN_MODE, StartupMode.FROM_TIMESTAMP);
+        }
+    }
+
+    /**
+     * Validate the {@link TableSchema} and {@link CoreOptions}.
+     *
+     * <p>TODO validate all items in schema and all keys in options.
+     *
+     * @param schema the schema to be validated
+     */
+    public static void validateTableSchema(TableSchema schema) {
+        CoreOptions options = new CoreOptions(schema.options());
+        if (options.startupMode() == StartupMode.FROM_TIMESTAMP) {
+            checkOptionExistInMode(options, SCAN_TIMESTAMP_MILLIS, StartupMode.FROM_TIMESTAMP);
+            checkOptionsConflict(options, SCAN_SNAPSHOT_ID, SCAN_TIMESTAMP_MILLIS);
+        } else if (options.startupMode() == StartupMode.FROM_SNAPSHOT) {
+            checkOptionExistInMode(options, SCAN_SNAPSHOT_ID, StartupMode.FROM_SNAPSHOT);
+            checkOptionsConflict(options, SCAN_TIMESTAMP_MILLIS, SCAN_SNAPSHOT_ID);
+        } else {
+            checkOptionNotExistInMode(options, SCAN_TIMESTAMP_MILLIS, options.startupMode());
+            checkOptionNotExistInMode(options, SCAN_SNAPSHOT_ID, options.startupMode());
+        }
+
+        Preconditions.checkArgument(
+                options.snapshotNumRetainMin() > 0,
+                SNAPSHOT_NUM_RETAINED_MIN.key() + " should be at least 1");
+        Preconditions.checkArgument(
+                options.snapshotNumRetainMin() <= options.snapshotNumRetainMax(),
+                SNAPSHOT_NUM_RETAINED_MIN.key()
+                        + " should not be larger than "
+                        + SNAPSHOT_NUM_RETAINED_MAX.key());
+
+        // Only changelog tables with primary keys support full compaction
+        if (options.changelogProducer() == ChangelogProducer.FULL_COMPACTION
+                && options.writeMode() == WriteMode.CHANGE_LOG
+                && schema.primaryKeys().isEmpty()) {
+            throw new UnsupportedOperationException(
+                    "Changelog table with full compaction must have primary keys");
+        }
+
+        // Check column names in schema
+        schema.fieldNames()
+                .forEach(
+                        f -> {
+                            checkState(
+                                    !SYSTEM_FIELD_NAMES.contains(f),
+                                    String.format(
+                                            "Field name[%s] in schema cannot be exist in [%s]",
+                                            f, SYSTEM_FIELD_NAMES.toString()));
+                            checkState(
+                                    !f.startsWith(KEY_FIELD_PREFIX),
+                                    String.format(
+                                            "Field name[%s] in schema cannot start with [%s]",
+                                            f, KEY_FIELD_PREFIX));
+                        });
+
+        // Cannot define any primary key in an append-only table.
+        if (!schema.primaryKeys().isEmpty() && Objects.equals(APPEND_ONLY, options.writeMode())) {
+            throw new TableException(
+                    "Cannot define any primary key in an append-only table. Set 'write-mode'='change-log' if "
+                            + "still want to keep the primary key definition.");
+        }
+    }
+
+    private static void checkOptionExistInMode(
+            CoreOptions options, ConfigOption<?> option, StartupMode startupMode) {
+        Preconditions.checkArgument(
+                options.options.contains(option),
+                String.format(
+                        "%s can not be null when you use %s for %s",
+                        option.key(), startupMode, SCAN_MODE.key()));
+    }
+
+    private static void checkOptionNotExistInMode(
+            CoreOptions options, ConfigOption<?> option, StartupMode startupMode) {
+        Preconditions.checkArgument(
+                !options.options.contains(option),
+                String.format(
+                        "%s must be null when you use %s for %s",
+                        option.key(), startupMode, SCAN_MODE.key()));
+    }
+
+    private static void checkOptionsConflict(
+            CoreOptions options, ConfigOption<?> illegalOption, ConfigOption<?> legalOption) {
+        Preconditions.checkArgument(
+                !options.options.contains(illegalOption),
+                String.format(
+                        "%s must be null when you set %s", illegalOption.key(), legalOption.key()));
+    }
+
     @Internal
     public static List<ConfigOption<?>> getOptions() {
         final Field[] fields = CoreOptions.class.getFields();
@@ -626,4 +890,27 @@ public class CoreOptions implements Serializable {
         }
         return list;
     }
+
+    @Internal
+    public static Set<String> getImmutableOptionKeys() {
+        final Field[] fields = CoreOptions.class.getFields();
+        final Set<String> immutableKeys = new HashSet<>(fields.length);
+        for (Field field : fields) {
+            if (ConfigOption.class.isAssignableFrom(field.getType())
+                    && field.getAnnotation(Immutable.class) != null) {
+                try {
+                    immutableKeys.add(((ConfigOption<?>) field.get(CoreOptions.class)).key());
+                } catch (IllegalAccessException e) {
+                    throw new RuntimeException(e);
+                }
+            }
+        }
+        return immutableKeys;
+    }
+
+    /** Annotation used on {@link ConfigOption} fields to exclude it from schema change. */
+    @Internal
+    @Target(ElementType.FIELD)
+    @Retention(RetentionPolicy.RUNTIME)
+    public @interface Immutable {}
 }
