@@ -18,6 +18,7 @@
 
 package org.apache.flink.table.store.file;
 
+import org.apache.flink.table.store.CoreOptions;
 import org.apache.flink.table.store.file.operation.FileStoreCommit;
 import org.apache.flink.table.store.file.operation.FileStoreExpire;
 import org.apache.flink.table.store.file.operation.FileStoreRead;
@@ -39,13 +40,15 @@ public interface FileStore<T> extends Serializable {
 
     RowType partitionType();
 
+    CoreOptions options();
+
     FileStoreScan newScan();
 
     FileStoreRead<T> newRead();
 
-    FileStoreWrite<T> newWrite();
+    FileStoreWrite<T> newWrite(String commitUser);
 
-    FileStoreCommit newCommit(String user);
+    FileStoreCommit newCommit(String commitUser);
 
     FileStoreExpire newExpire();
 }
