@@ -20,6 +20,9 @@ package org.apache.flink.table.store.file.catalog;
 
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.core.fs.Path;
+import org.apache.flink.table.store.table.TableType;
+
+import static org.apache.flink.table.store.CatalogOptions.TABLE_TYPE;
 
 /** Factory to create {@link FileSystemCatalog}. */
 public class FileSystemCatalogFactory implements CatalogFactory {
@@ -33,6 +36,10 @@ public class FileSystemCatalogFactory implements CatalogFactory {
 
     @Override
     public Catalog create(String warehouse, Configuration options) {
+        if (!TableType.MANAGED.equals(options.get(TABLE_TYPE))) {
+            throw new IllegalArgumentException(
+                    "Only managed table is supported in File system catalog.");
+        }
         return new FileSystemCatalog(new Path(warehouse));
     }
 }
