@@ -21,44 +21,40 @@ package org.apache.flink.table.store.file.manifest;
 import org.apache.flink.core.io.SimpleVersionedSerializer;
 import org.apache.flink.core.memory.DataInputDeserializer;
 import org.apache.flink.core.memory.DataOutputViewStreamWrapper;
-import org.apache.flink.table.data.binary.BinaryRowData;
-import org.apache.flink.table.store.file.data.DataFileMeta;
-import org.apache.flink.table.store.file.data.DataFileMetaSerializer;
+import org.apache.flink.table.store.table.sink.FileCommittable;
+import org.apache.flink.table.store.table.sink.FileCommittableSerializer;
 
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
-import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
-import static org.apache.flink.table.store.file.utils.SerializationUtils.deserializeBinaryRow;
-import static org.apache.flink.table.store.file.utils.SerializationUtils.serializeBinaryRow;
-
 /** {@link SimpleVersionedSerializer} for {@link ManifestCommittable}. */
 public class ManifestCommittableSerializer
         implements SimpleVersionedSerializer<ManifestCommittable> {
 
-    private final DataFileMetaSerializer dataFileSerializer;
+    private static final int CURRENT_VERSION = 2;
+
+    private final FileCommittableSerializer fileCommittableSerializer;
 
     public ManifestCommittableSerializer() {
-        this.dataFileSerializer = new DataFileMetaSerializer();
+        this.fileCommittableSerializer = new FileCommittableSerializer();
     }
 
     @Override
     public int getVersion() {
-        return 1;
+        return CURRENT_VERSION;
     }
 
     @Override
     public byte[] serialize(ManifestCommittable obj) throws IOException {
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         DataOutputViewStreamWrapper view = new DataOutputViewStreamWrapper(out);
-        view.writeUTF(obj.identifier());
+        view.writeLong(obj.identifier());
         serializeOffsets(view, obj.logOffsets());
-        serializeFiles(view, obj.newFiles());
-        serializeFiles(view, obj.compactBefore());
-        serializeFiles(view, obj.compactAfter());
+        view.writeInt(fileCommittableSerializer.getVersion());
+        fileCommittableSerializer.serializeList(obj.fileCommittables(), view);
         return out.toByteArray();
     }
 
@@ -71,6 +67,27 @@ public class ManifestCommittableSerializer
         }
     }
 
+    @Override
+    public ManifestCommittable deserialize(int version, byte[] serialized) throws IOException {
+        if (version != CURRENT_VERSION) {
+            throw new UnsupportedOperationException(
+                    "Expecting ManifestCommittable version to be "
+                            + CURRENT_VERSION
+                            + ", but found "
+                            + version
+                            + ".\nManifestCommittable is not a compatible data structure. "
+                            + "Please restart the job afresh (do not recover from savepoint).");
+        }
+
+        DataInputDeserializer view = new DataInputDeserializer(serialized);
+        long identifier = view.readLong();
+        Map<Integer, Long> offsets = deserializeOffsets(view);
+        int fileCommittableSerializerVersion = view.readInt();
+        List<FileCommittable> fileCommittables =
+                fileCommittableSerializer.deserializeList(fileCommittableSerializerVersion, view);
+        return new ManifestCommittable(identifier, offsets, fileCommittables);
+    }
+
     private Map<Integer, Long> deserializeOffsets(DataInputDeserializer view) throws IOException {
         int size = view.readInt();
         Map<Integer, Long> offsets = new HashMap<>(size);
@@ -79,55 +96,4 @@ public class ManifestCommittableSerializer
         }
         return offsets;
     }
-
-    private void serializeFiles(
-            DataOutputViewStreamWrapper view,
-            Map<BinaryRowData, Map<Integer, List<DataFileMeta>>> files)
-            throws IOException {
-        view.writeInt(files.size());
-        for (Map.Entry<BinaryRowData, Map<Integer, List<DataFileMeta>>> entry : files.entrySet()) {
-            serializeBinaryRow(entry.getKey(), view);
-            view.writeInt(entry.getValue().size());
-            for (Map.Entry<Integer, List<DataFileMeta>> bucketEntry : entry.getValue().entrySet()) {
-                view.writeInt(bucketEntry.getKey());
-                view.writeInt(bucketEntry.getValue().size());
-                for (DataFileMeta file : bucketEntry.getValue()) {
-                    dataFileSerializer.serialize(file, view);
-                }
-            }
-        }
-    }
-
-    private Map<BinaryRowData, Map<Integer, List<DataFileMeta>>> deserializeFiles(
-            DataInputDeserializer view) throws IOException {
-        int partNumber = view.readInt();
-        Map<BinaryRowData, Map<Integer, List<DataFileMeta>>> files = new HashMap<>();
-        for (int i = 0; i < partNumber; i++) {
-            BinaryRowData part = deserializeBinaryRow(view);
-            int bucketNumber = view.readInt();
-            Map<Integer, List<DataFileMeta>> bucketMap = new HashMap<>();
-            files.put(part, bucketMap);
-            for (int j = 0; j < bucketNumber; j++) {
-                int bucket = view.readInt();
-                int fileNumber = view.readInt();
-                List<DataFileMeta> fileMetas = new ArrayList<>();
-                bucketMap.put(bucket, fileMetas);
-                for (int k = 0; k < fileNumber; k++) {
-                    fileMetas.add(dataFileSerializer.deserialize(view));
-                }
-            }
-        }
-        return files;
-    }
-
-    @Override
-    public ManifestCommittable deserialize(int version, byte[] serialized) throws IOException {
-        DataInputDeserializer view = new DataInputDeserializer(serialized);
-        return new ManifestCommittable(
-                view.readUTF(),
-                deserializeOffsets(view),
-                deserializeFiles(view),
-                deserializeFiles(view),
-                deserializeFiles(view));
-    }
 }
