@@ -24,33 +24,24 @@ import org.apache.flink.connector.file.src.FileSourceSplit;
 import org.apache.flink.connector.file.src.reader.BulkFormat;
 import org.apache.flink.core.fs.Path;
 import org.apache.flink.table.data.RowData;
+import org.apache.flink.table.store.file.io.RollingFileWriter;
+import org.apache.flink.table.store.file.io.SingleFileWriter;
 import org.apache.flink.table.store.file.schema.SchemaManager;
 import org.apache.flink.table.store.file.stats.FieldStatsArraySerializer;
 import org.apache.flink.table.store.file.utils.FileStorePathFactory;
 import org.apache.flink.table.store.file.utils.FileUtils;
 import org.apache.flink.table.store.file.utils.VersionedObjectSerializer;
-import org.apache.flink.table.store.file.writer.BaseFileWriter;
-import org.apache.flink.table.store.file.writer.FileWriter;
-import org.apache.flink.table.store.file.writer.Metric;
-import org.apache.flink.table.store.file.writer.MetricFileWriter;
-import org.apache.flink.table.store.file.writer.RollingFileWriter;
 import org.apache.flink.table.store.format.FieldStatsCollector;
 import org.apache.flink.table.store.format.FileFormat;
-import org.apache.flink.table.store.format.FileStatsExtractor;
 import org.apache.flink.table.types.logical.RowType;
 
 import org.apache.flink.shaded.guava30.com.google.common.collect.Iterables;
 
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
 import java.io.IOException;
-import java.io.UncheckedIOException;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Queue;
-import java.util.function.Supplier;
 
 /**
  * This file includes several {@link ManifestEntry}s, representing the additional changes since last
@@ -58,26 +49,22 @@ import java.util.function.Supplier;
  */
 public class ManifestFile {
 
-    private static final Logger LOG = LoggerFactory.getLogger(ManifestFile.class);
-
     private final SchemaManager schemaManager;
     private final long schemaId;
     private final RowType partitionType;
     private final ManifestEntrySerializer serializer;
     private final BulkFormat<RowData, FileSourceSplit> readerFactory;
+    private final BulkWriter.Factory<RowData> writerFactory;
     private final FileStorePathFactory pathFactory;
     private final long suggestedFileSize;
-    private final FileWriter.Factory<ManifestEntry, Metric> fileWriterFactory;
 
     private ManifestFile(
             SchemaManager schemaManager,
             long schemaId,
             RowType partitionType,
-            RowType entryType,
             ManifestEntrySerializer serializer,
             BulkFormat<RowData, FileSourceSplit> readerFactory,
             BulkWriter.Factory<RowData> writerFactory,
-            FileStatsExtractor fileStatsExtractor,
             FileStorePathFactory pathFactory,
             long suggestedFileSize) {
         this.schemaManager = schemaManager;
@@ -85,13 +72,9 @@ public class ManifestFile {
         this.partitionType = partitionType;
         this.serializer = serializer;
         this.readerFactory = readerFactory;
+        this.writerFactory = writerFactory;
         this.pathFactory = pathFactory;
         this.suggestedFileSize = suggestedFileSize;
-
-        // Initialize the metric file writer factory to write manifest entry and generate metrics.
-        this.fileWriterFactory =
-                MetricFileWriter.createFactory(
-                        writerFactory, serializer::toRow, entryType, fileStatsExtractor);
     }
 
     @VisibleForTesting
@@ -132,35 +115,33 @@ public class ManifestFile {
      * <p>NOTE: This method is atomic.
      */
     public List<ManifestFileMeta> write(List<ManifestEntry> entries) {
-
-        ManifestRollingWriter rollingWriter = createManifestRollingWriter(suggestedFileSize);
-        try (ManifestRollingWriter writer = rollingWriter) {
+        RollingFileWriter<ManifestEntry, ManifestFileMeta> writer =
+                new RollingFileWriter<>(
+                        () -> new ManifestEntryWriter(writerFactory, pathFactory.newManifestFile()),
+                        suggestedFileSize);
+        try {
             writer.write(entries);
-
+            writer.close();
         } catch (Exception e) {
-            LOG.warn("Exception occurs when writing manifest files. Cleaning up.", e);
-
-            rollingWriter.abort();
             throw new RuntimeException(e);
         }
-
-        return rollingWriter.result();
+        return writer.result();
     }
 
     public void delete(String fileName) {
         FileUtils.deleteOrWarn(pathFactory.toManifestFilePath(fileName));
     }
 
-    private class ManifestEntryWriter extends BaseFileWriter<ManifestEntry, ManifestFileMeta> {
+    private class ManifestEntryWriter extends SingleFileWriter<ManifestEntry, ManifestFileMeta> {
 
         private final FieldStatsCollector partitionStatsCollector;
         private final FieldStatsArraySerializer partitionStatsSerializer;
+
         private long numAddedFiles = 0;
         private long numDeletedFiles = 0;
 
-        ManifestEntryWriter(FileWriter.Factory<ManifestEntry, Metric> writerFactory, Path path)
-                throws IOException {
-            super(writerFactory, path);
+        ManifestEntryWriter(BulkWriter.Factory<RowData> factory, Path path) {
+            super(factory, path, serializer::toRow);
 
             this.partitionStatsCollector = new FieldStatsCollector(partitionType);
             this.partitionStatsSerializer = new FieldStatsArraySerializer(partitionType);
@@ -185,9 +166,7 @@ public class ManifestFile {
         }
 
         @Override
-        protected ManifestFileMeta createResult(Path path, Metric ignore) throws IOException {
-            // The input metric will be ignored because it includes all the column's stats, rather
-            // than the partition stats.
+        public ManifestFileMeta result() throws IOException {
             return new ManifestFileMeta(
                     path.getName(),
                     path.getFileSystem().getFileStatus(path).getLen(),
@@ -198,29 +177,6 @@ public class ManifestFile {
         }
     }
 
-    private static class ManifestRollingWriter
-            extends RollingFileWriter<ManifestEntry, ManifestFileMeta> {
-
-        public ManifestRollingWriter(
-                Supplier<ManifestEntryWriter> writerFactory, long targetFileSize) {
-            super(writerFactory, targetFileSize);
-        }
-    }
-
-    private Supplier<ManifestEntryWriter> createWriterFactory() {
-        return () -> {
-            try {
-                return new ManifestEntryWriter(fileWriterFactory, pathFactory.newManifestFile());
-            } catch (IOException e) {
-                throw new UncheckedIOException(e);
-            }
-        };
-    }
-
-    private ManifestRollingWriter createManifestRollingWriter(long targetFileSize) {
-        return new ManifestRollingWriter(createWriterFactory(), targetFileSize);
-    }
-
     /**
      * Creator of {@link ManifestFile}. It reueses {@link BulkFormat} and {@link BulkWriter.Factory}
      * from {@link FileFormat}.
@@ -255,11 +211,9 @@ public class ManifestFile {
                     schemaManager,
                     schemaId,
                     partitionType,
-                    entryType,
                     new ManifestEntrySerializer(),
                     fileFormat.createReaderFactory(entryType),
                     fileFormat.createWriterFactory(entryType),
-                    fileFormat.createStatsExtractor(entryType).orElse(null),
                     pathFactory,
                     suggestedFileSize);
         }
