@@ -19,23 +19,25 @@
 package org.apache.flink.table.store.file.mergetree;
 
 import org.apache.flink.annotation.VisibleForTesting;
+import org.apache.flink.runtime.io.disk.iomanager.IOManager;
 import org.apache.flink.table.data.RowData;
 import org.apache.flink.table.runtime.util.MemorySegmentPool;
 import org.apache.flink.table.store.CoreOptions.ChangelogProducer;
 import org.apache.flink.table.store.file.KeyValue;
 import org.apache.flink.table.store.file.compact.CompactManager;
 import org.apache.flink.table.store.file.compact.CompactResult;
-import org.apache.flink.table.store.file.data.DataFileMeta;
-import org.apache.flink.table.store.file.data.DataFileWriter;
+import org.apache.flink.table.store.file.io.CompactIncrement;
+import org.apache.flink.table.store.file.io.DataFileMeta;
+import org.apache.flink.table.store.file.io.KeyValueFileWriterFactory;
+import org.apache.flink.table.store.file.io.NewFilesIncrement;
+import org.apache.flink.table.store.file.io.RollingFileWriter;
 import org.apache.flink.table.store.file.memory.MemoryOwner;
 import org.apache.flink.table.store.file.mergetree.compact.MergeFunction;
-import org.apache.flink.table.store.file.writer.RecordWriter;
+import org.apache.flink.table.store.file.utils.RecordWriter;
 import org.apache.flink.table.types.logical.RowType;
-import org.apache.flink.util.CloseableIterator;
 
 import java.util.ArrayList;
 import java.util.Comparator;
-import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.LinkedHashSet;
 import java.util.List;
@@ -43,65 +45,60 @@ import java.util.Optional;
 import java.util.Set;
 import java.util.stream.Collectors;
 
-/** A {@link RecordWriter} to write records and generate {@link Increment}. */
+/** A {@link RecordWriter} to write records and generate {@link CompactIncrement}. */
 public class MergeTreeWriter implements RecordWriter<KeyValue>, MemoryOwner {
 
+    private final boolean writeBufferSpillable;
+    private final int sortMaxFan;
+    private final IOManager ioManager;
+
     private final RowType keyType;
-
     private final RowType valueType;
-
     private final CompactManager compactManager;
-
-    private final Levels levels;
-
     private final Comparator<RowData> keyComparator;
-
-    private final MergeFunction mergeFunction;
-
-    private final DataFileWriter dataFileWriter;
-
+    private final MergeFunction<KeyValue> mergeFunction;
+    private final KeyValueFileWriterFactory writerFactory;
     private final boolean commitForceCompact;
-
-    private final int numSortedRunStopTrigger;
-
     private final ChangelogProducer changelogProducer;
 
     private final LinkedHashSet<DataFileMeta> newFiles;
-
+    private final LinkedHashSet<DataFileMeta> newFilesChangelog;
     private final LinkedHashMap<String, DataFileMeta> compactBefore;
-
     private final LinkedHashSet<DataFileMeta> compactAfter;
+    private final LinkedHashSet<DataFileMeta> compactChangelog;
 
     private long newSequenceNumber;
-
-    private MemTable memTable;
+    private WriteBuffer writeBuffer;
 
     public MergeTreeWriter(
-            RowType keyType,
-            RowType valueType,
+            boolean writeBufferSpillable,
+            int sortMaxFan,
+            IOManager ioManager,
             CompactManager compactManager,
-            Levels levels,
             long maxSequenceNumber,
             Comparator<RowData> keyComparator,
-            MergeFunction mergeFunction,
-            DataFileWriter dataFileWriter,
+            MergeFunction<KeyValue> mergeFunction,
+            KeyValueFileWriterFactory writerFactory,
             boolean commitForceCompact,
-            int numSortedRunStopTrigger,
             ChangelogProducer changelogProducer) {
-        this.keyType = keyType;
-        this.valueType = valueType;
+        this.writeBufferSpillable = writeBufferSpillable;
+        this.sortMaxFan = sortMaxFan;
+        this.ioManager = ioManager;
+        this.keyType = writerFactory.keyType();
+        this.valueType = writerFactory.valueType();
         this.compactManager = compactManager;
-        this.levels = levels;
         this.newSequenceNumber = maxSequenceNumber + 1;
         this.keyComparator = keyComparator;
         this.mergeFunction = mergeFunction;
-        this.dataFileWriter = dataFileWriter;
+        this.writerFactory = writerFactory;
         this.commitForceCompact = commitForceCompact;
-        this.numSortedRunStopTrigger = numSortedRunStopTrigger;
         this.changelogProducer = changelogProducer;
+
         this.newFiles = new LinkedHashSet<>();
+        this.newFilesChangelog = new LinkedHashSet<>();
         this.compactBefore = new LinkedHashMap<>();
         this.compactAfter = new LinkedHashSet<>();
+        this.compactChangelog = new LinkedHashSet<>();
     }
 
     private long newSequenceNumber() {
@@ -109,13 +106,20 @@ public class MergeTreeWriter implements RecordWriter<KeyValue>, MemoryOwner {
     }
 
     @VisibleForTesting
-    Levels levels() {
-        return levels;
+    CompactManager compactManager() {
+        return compactManager;
     }
 
     @Override
     public void setMemoryPool(MemorySegmentPool memoryPool) {
-        this.memTable = new SortBufferMemTable(keyType, valueType, memoryPool);
+        this.writeBuffer =
+                new SortBufferWriteBuffer(
+                        keyType,
+                        valueType,
+                        memoryPool,
+                        writeBufferSpillable,
+                        sortMaxFan,
+                        ioManager);
     }
 
     @Override
@@ -124,84 +128,121 @@ public class MergeTreeWriter implements RecordWriter<KeyValue>, MemoryOwner {
                 kv.sequenceNumber() == KeyValue.UNKNOWN_SEQUENCE
                         ? newSequenceNumber()
                         : kv.sequenceNumber();
-        boolean success = memTable.put(sequenceNumber, kv.valueKind(), kv.key(), kv.value());
+        boolean success = writeBuffer.put(sequenceNumber, kv.valueKind(), kv.key(), kv.value());
         if (!success) {
-            flushMemory();
-            success = memTable.put(sequenceNumber, kv.valueKind(), kv.key(), kv.value());
+            flushWriteBuffer(false, false);
+            success = writeBuffer.put(sequenceNumber, kv.valueKind(), kv.key(), kv.value());
             if (!success) {
                 throw new RuntimeException("Mem table is too small to hold a single element.");
             }
         }
     }
 
+    @Override
+    public void compact(boolean fullCompaction) throws Exception {
+        flushWriteBuffer(true, fullCompaction);
+    }
+
+    @Override
+    public void addNewFiles(List<DataFileMeta> files) {
+        files.forEach(compactManager::addNewFile);
+    }
+
     @Override
     public long memoryOccupancy() {
-        return memTable.memoryOccupancy();
+        return writeBuffer.memoryOccupancy();
     }
 
     @Override
     public void flushMemory() throws Exception {
-        if (memTable.size() > 0) {
-            if (levels.numberOfSortedRuns() > numSortedRunStopTrigger) {
-                // stop writing, wait for compaction finished
-                finishCompaction(true);
-            }
-            List<String> extraFiles = new ArrayList<>();
-            if (changelogProducer == ChangelogProducer.INPUT) {
-                extraFiles.add(
-                        dataFileWriter
-                                .writeLevel0Changelog(
-                                        CloseableIterator.adapterForIterator(
-                                                memTable.rawIterator()))
-                                .getName());
-            }
-            boolean success = false;
-            try {
-                Iterator<KeyValue> iterator = memTable.mergeIterator(keyComparator, mergeFunction);
-                success =
-                        dataFileWriter
-                                .writeLevel0(CloseableIterator.adapterForIterator(iterator))
-                                .map(
-                                        file -> {
-                                            DataFileMeta fileMeta = file.copy(extraFiles);
-                                            newFiles.add(fileMeta);
-                                            levels.addLevel0File(fileMeta);
-                                            return true;
-                                        })
-                                .orElse(false);
-            } finally {
-                if (!success) {
-                    extraFiles.forEach(dataFileWriter::delete);
-                }
-            }
-            memTable.clear();
-            submitCompaction();
+        boolean success = writeBuffer.flushMemory();
+        if (!success) {
+            flushWriteBuffer(false, false);
         }
     }
 
+    private void flushWriteBuffer(boolean waitForLatestCompaction, boolean forcedFullCompaction)
+            throws Exception {
+        if (writeBuffer.size() > 0) {
+            if (compactManager.shouldWaitCompaction()) {
+                waitForLatestCompaction = true;
+            }
+
+            final RollingFileWriter<KeyValue, DataFileMeta> changelogWriter =
+                    changelogProducer == ChangelogProducer.INPUT
+                            ? writerFactory.createRollingChangelogFileWriter(0)
+                            : null;
+            final RollingFileWriter<KeyValue, DataFileMeta> dataWriter =
+                    writerFactory.createRollingMergeTreeFileWriter(0);
+
+            try {
+                writeBuffer.forEach(
+                        keyComparator,
+                        mergeFunction,
+                        changelogWriter == null ? null : changelogWriter::write,
+                        dataWriter::write);
+            } finally {
+                if (changelogWriter != null) {
+                    changelogWriter.close();
+                }
+                dataWriter.close();
+            }
+
+            if (changelogWriter != null) {
+                newFilesChangelog.addAll(changelogWriter.result());
+            }
+
+            for (DataFileMeta fileMeta : dataWriter.result()) {
+                newFiles.add(fileMeta);
+                compactManager.addNewFile(fileMeta);
+            }
+
+            writeBuffer.clear();
+        }
+
+        trySyncLatestCompaction(waitForLatestCompaction);
+        compactManager.triggerCompaction(forcedFullCompaction);
+    }
+
     @Override
-    public Increment prepareCommit(boolean endOfInput) throws Exception {
-        flushMemory();
-        boolean blocking = endOfInput || commitForceCompact;
-        finishCompaction(blocking);
+    public CommitIncrement prepareCommit(boolean blocking) throws Exception {
+        flushWriteBuffer(false, false);
+        trySyncLatestCompaction(blocking || commitForceCompact);
         return drainIncrement();
     }
 
     @Override
     public void sync() throws Exception {
-        finishCompaction(true);
+        trySyncLatestCompaction(true);
     }
 
-    private Increment drainIncrement() {
-        Increment increment =
-                new Increment(
-                        new ArrayList<>(newFiles),
+    private CommitIncrement drainIncrement() {
+        NewFilesIncrement newFilesIncrement =
+                new NewFilesIncrement(
+                        new ArrayList<>(newFiles), new ArrayList<>(newFilesChangelog));
+        CompactIncrement compactIncrement =
+                new CompactIncrement(
                         new ArrayList<>(compactBefore.values()),
-                        new ArrayList<>(compactAfter));
+                        new ArrayList<>(compactAfter),
+                        new ArrayList<>(compactChangelog));
+
         newFiles.clear();
+        newFilesChangelog.clear();
         compactBefore.clear();
         compactAfter.clear();
-        return increment;
+        compactChangelog.clear();
+
+        return new CommitIncrement() {
+            @Override
+            public NewFilesIncrement newFilesIncrement() {
+                return newFilesIncrement;
+            }
+
+            @Override
+            public CompactIncrement compactIncrement() {
+                return compactIncrement;
+            }
+        };
     }
 
     private void updateCompactResult(CompactResult result) {
@@ -216,35 +257,36 @@ public class MergeTreeWriter implements RecordWriter<KeyValue>, MemoryOwner {
                 // 2. This file is not the input of upgraded.
                 if (!compactBefore.containsKey(file.fileName())
                         && !afterFiles.contains(file.fileName())) {
-                    dataFileWriter.delete(file);
+                    writerFactory.deleteFile(file.fileName());
                 }
             } else {
                 compactBefore.put(file.fileName(), file);
             }
         }
         compactAfter.addAll(result.after());
+        compactChangelog.addAll(result.changelog());
     }
 
-    private void submitCompaction() throws Exception {
-        finishCompaction(false);
-        if (compactManager.isCompactionFinished()) {
-            compactManager.submitCompaction();
-        }
-    }
-
-    private void finishCompaction(boolean blocking) throws Exception {
-        Optional<CompactResult> result = compactManager.finishCompaction(blocking);
+    private void trySyncLatestCompaction(boolean blocking) throws Exception {
+        Optional<CompactResult> result = compactManager.getCompactionResult(blocking);
         result.ifPresent(this::updateCompactResult);
     }
 
     @Override
-    public List<DataFileMeta> close() throws Exception {
+    public void close() throws Exception {
         // cancel compaction so that it does not block job cancelling
         compactManager.cancelCompaction();
         sync();
 
         // delete temporary files
         List<DataFileMeta> delete = new ArrayList<>(newFiles);
+        newFiles.clear();
+
+        for (DataFileMeta file : newFilesChangelog) {
+            writerFactory.deleteFile(file.fileName());
+        }
+        newFilesChangelog.clear();
+
         for (DataFileMeta file : compactAfter) {
             // upgrade file is required by previous snapshot, so we should ensure that this file is
             // not the output of upgraded.
@@ -252,11 +294,16 @@ public class MergeTreeWriter implements RecordWriter<KeyValue>, MemoryOwner {
                 delete.add(file);
             }
         }
-        for (DataFileMeta file : delete) {
-            dataFileWriter.delete(file);
-        }
-        newFiles.clear();
+
         compactAfter.clear();
-        return delete;
+
+        for (DataFileMeta file : compactChangelog) {
+            writerFactory.deleteFile(file.fileName());
+        }
+        compactChangelog.clear();
+
+        for (DataFileMeta file : delete) {
+            writerFactory.deleteFile(file.fileName());
+        }
     }
 }
