@@ -18,26 +18,24 @@
 
 package org.apache.flink.table.store.file.mergetree.compact;
 
-import org.apache.flink.table.data.RowData;
 import org.apache.flink.table.store.file.KeyValue;
 
 import javax.annotation.Nullable;
 
-import java.io.Serializable;
-
-/** Merge function to merge multiple {@link KeyValue}s. */
-public interface MergeFunction extends Serializable {
+/**
+ * Merge function to merge multiple {@link KeyValue}s.
+ *
+ * @param <T> result type
+ */
+public interface MergeFunction<T> {
 
     /** Reset the merge function to its default state. */
     void reset();
 
-    /** Add the given {@link RowData} to the merge function. */
-    void add(RowData value);
+    /** Add the given {@link KeyValue} to the merge function. */
+    void add(KeyValue kv);
 
     /** Get current merged value. Return null if this merged result should be skipped. */
     @Nullable
-    RowData getValue();
-
-    /** Create a new merge function object with the same functionality as this one. */
-    MergeFunction copy();
+    T getResult();
 }
