@@ -20,7 +20,7 @@ package org.apache.flink.table.store.file.operation;
 
 import org.apache.flink.table.store.file.predicate.Predicate;
 import org.apache.flink.table.store.file.utils.RecordReader;
-import org.apache.flink.table.store.table.source.Split;
+import org.apache.flink.table.store.table.source.DataSplit;
 
 import java.io.IOException;
 
@@ -34,5 +34,5 @@ public interface FileStoreRead<T> {
     FileStoreRead<T> withFilter(Predicate predicate);
 
     /** Create a {@link RecordReader} from split. */
-    RecordReader<T> createReader(Split split) throws IOException;
+    RecordReader<T> createReader(DataSplit split) throws IOException;
 }
