@@ -24,27 +24,29 @@ import org.apache.flink.core.fs.Path;
 import org.apache.flink.table.data.binary.BinaryRowData;
 import org.apache.flink.table.runtime.typeutils.RowDataSerializer;
 import org.apache.flink.table.store.CoreOptions;
-import org.apache.flink.table.store.file.data.DataFileMeta;
+import org.apache.flink.table.store.file.io.DataFileMeta;
 import org.apache.flink.table.store.file.manifest.ManifestCommittable;
 import org.apache.flink.table.store.file.manifest.ManifestEntry;
 import org.apache.flink.table.store.file.manifest.ManifestFileMeta;
 import org.apache.flink.table.store.file.manifest.ManifestList;
 import org.apache.flink.table.store.file.memory.HeapMemorySegmentPool;
 import org.apache.flink.table.store.file.memory.MemoryOwner;
-import org.apache.flink.table.store.file.mergetree.Increment;
-import org.apache.flink.table.store.file.mergetree.compact.MergeFunction;
+import org.apache.flink.table.store.file.mergetree.compact.MergeFunctionFactory;
+import org.apache.flink.table.store.file.operation.AbstractFileStoreWrite;
 import org.apache.flink.table.store.file.operation.FileStoreCommit;
 import org.apache.flink.table.store.file.operation.FileStoreCommitImpl;
 import org.apache.flink.table.store.file.operation.FileStoreExpireImpl;
 import org.apache.flink.table.store.file.operation.FileStoreRead;
 import org.apache.flink.table.store.file.operation.FileStoreScan;
-import org.apache.flink.table.store.file.operation.FileStoreWrite;
+import org.apache.flink.table.store.file.operation.ScanKind;
+import org.apache.flink.table.store.file.schema.KeyValueFieldsExtractor;
 import org.apache.flink.table.store.file.schema.SchemaManager;
 import org.apache.flink.table.store.file.utils.FileStorePathFactory;
 import org.apache.flink.table.store.file.utils.RecordReaderIterator;
+import org.apache.flink.table.store.file.utils.RecordWriter;
 import org.apache.flink.table.store.file.utils.SnapshotManager;
-import org.apache.flink.table.store.file.writer.RecordWriter;
-import org.apache.flink.table.store.table.source.Split;
+import org.apache.flink.table.store.table.sink.FileCommittable;
+import org.apache.flink.table.store.table.source.DataSplit;
 import org.apache.flink.table.types.logical.RowType;
 
 import org.slf4j.Logger;
@@ -59,7 +61,6 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
-import java.util.Random;
 import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.ExecutorService;
@@ -83,34 +84,9 @@ public class TestFileStore extends KeyValueFileStore {
     private final String root;
     private final RowDataSerializer keySerializer;
     private final RowDataSerializer valueSerializer;
-    private final String user;
+    private final String commitUser;
 
-    public static TestFileStore create(
-            String format,
-            String root,
-            int numBuckets,
-            RowType partitionType,
-            RowType keyType,
-            RowType valueType,
-            MergeFunction mergeFunction) {
-        Configuration conf = new Configuration();
-
-        conf.set(CoreOptions.WRITE_BUFFER_SIZE, WRITE_BUFFER_SIZE);
-        conf.set(CoreOptions.PAGE_SIZE, PAGE_SIZE);
-        conf.set(CoreOptions.TARGET_FILE_SIZE, MemorySize.parse("1 kb"));
-
-        conf.set(
-                CoreOptions.MANIFEST_TARGET_FILE_SIZE,
-                MemorySize.parse((ThreadLocalRandom.current().nextInt(16) + 1) + "kb"));
-
-        conf.set(CoreOptions.FILE_FORMAT, format);
-        conf.set(CoreOptions.MANIFEST_FORMAT, format);
-        conf.set(CoreOptions.PATH, root);
-        conf.set(CoreOptions.BUCKET, numBuckets);
-
-        return new TestFileStore(
-                root, new CoreOptions(conf), partitionType, keyType, valueType, mergeFunction);
-    }
+    private long commitIdentifier;
 
     private TestFileStore(
             String root,
@@ -118,7 +94,8 @@ public class TestFileStore extends KeyValueFileStore {
             RowType partitionType,
             RowType keyType,
             RowType valueType,
-            MergeFunction mergeFunction) {
+            KeyValueFieldsExtractor keyValueFieldsExtractor,
+            MergeFunctionFactory<KeyValue> mfFactory) {
         super(
                 new SchemaManager(options.path()),
                 0L,
@@ -127,15 +104,18 @@ public class TestFileStore extends KeyValueFileStore {
                 keyType,
                 keyType,
                 valueType,
-                mergeFunction);
+                keyValueFieldsExtractor,
+                mfFactory);
         this.root = root;
         this.keySerializer = new RowDataSerializer(keyType);
         this.valueSerializer = new RowDataSerializer(valueType);
-        this.user = UUID.randomUUID().toString();
+        this.commitUser = UUID.randomUUID().toString();
+
+        this.commitIdentifier = 0L;
     }
 
     public FileStoreCommitImpl newCommit() {
-        return super.newCommit(user);
+        return super.newCommit(commitUser);
     }
 
     public FileStoreExpireImpl newExpire(
@@ -197,10 +177,10 @@ public class TestFileStore extends KeyValueFileStore {
             Function<KeyValue, BinaryRowData> partitionCalculator,
             Function<KeyValue, Integer> bucketCalculator,
             boolean emptyWriter,
-            String identifier,
+            Long identifier,
             BiConsumer<FileStoreCommit, ManifestCommittable> commitFunction)
             throws Exception {
-        FileStoreWrite<KeyValue> write = newWrite();
+        AbstractFileStoreWrite<KeyValue> write = newWrite(commitUser);
         Map<BinaryRowData, Map<Integer, RecordWriter<KeyValue>>> writers = new HashMap<>();
         for (KeyValue kv : kvs) {
             BinaryRowData partition = partitionCalculator.apply(kv);
@@ -213,10 +193,12 @@ public class TestFileStore extends KeyValueFileStore {
                                     ExecutorService service = Executors.newSingleThreadExecutor();
                                     RecordWriter<KeyValue> writer =
                                             emptyWriter
-                                                    ? write.createEmptyWriter(
-                                                            partition, bucket, service)
-                                                    : write.createWriter(
-                                                            partition, bucket, service);
+                                                    ? write.createEmptyWriterContainer(
+                                                                    partition, bucket, service)
+                                                            .writer
+                                                    : write.createWriterContainer(
+                                                                    partition, bucket, service)
+                                                            .writer;
                                     ((MemoryOwner) writer)
                                             .setMemoryPool(
                                                     new HeapMemorySegmentPool(
@@ -230,17 +212,21 @@ public class TestFileStore extends KeyValueFileStore {
                     .write(kv);
         }
 
-        FileStoreCommit commit = newCommit(user);
+        FileStoreCommit commit = newCommit(commitUser);
         ManifestCommittable committable =
-                new ManifestCommittable(
-                        identifier == null ? String.valueOf(new Random().nextLong()) : identifier);
+                new ManifestCommittable(identifier == null ? commitIdentifier++ : identifier);
         for (Map.Entry<BinaryRowData, Map<Integer, RecordWriter<KeyValue>>> entryWithPartition :
                 writers.entrySet()) {
             for (Map.Entry<Integer, RecordWriter<KeyValue>> entryWithBucket :
                     entryWithPartition.getValue().entrySet()) {
-                Increment increment = entryWithBucket.getValue().prepareCommit(emptyWriter);
+                RecordWriter.CommitIncrement increment =
+                        entryWithBucket.getValue().prepareCommit(emptyWriter);
                 committable.addFileCommittable(
-                        entryWithPartition.getKey(), entryWithBucket.getKey(), increment);
+                        new FileCommittable(
+                                entryWithPartition.getKey(),
+                                entryWithBucket.getKey(),
+                                increment.newFilesIncrement(),
+                                increment.compactIncrement()));
             }
         }
 
@@ -276,13 +262,33 @@ public class TestFileStore extends KeyValueFileStore {
         return snapshots;
     }
 
-    public List<KeyValue> readKvsFromSnapshot(long snapshotId) throws IOException {
+    public List<KeyValue> readKvsFromSnapshot(long snapshotId) throws Exception {
         List<ManifestEntry> entries = newScan().withSnapshot(snapshotId).plan().files();
-        return readKvsFromManifestEntries(entries);
+        return readKvsFromManifestEntries(entries, false);
     }
 
-    public List<KeyValue> readKvsFromManifestEntries(List<ManifestEntry> entries)
-            throws IOException {
+    public List<KeyValue> readAllChangelogUntilSnapshot(long endInclusive) throws Exception {
+        List<KeyValue> result = new ArrayList<>();
+        for (long snapshotId = Snapshot.FIRST_SNAPSHOT_ID;
+                snapshotId <= endInclusive;
+                snapshotId++) {
+            List<ManifestEntry> entries =
+                    newScan()
+                            .withKind(
+                                    options.changelogProducer()
+                                                    == CoreOptions.ChangelogProducer.NONE
+                                            ? ScanKind.DELTA
+                                            : ScanKind.CHANGELOG)
+                            .withSnapshot(snapshotId)
+                            .plan()
+                            .files();
+            result.addAll(readKvsFromManifestEntries(entries, true));
+        }
+        return result;
+    }
+
+    public List<KeyValue> readKvsFromManifestEntries(
+            List<ManifestEntry> entries, boolean isIncremental) throws Exception {
         if (LOG.isDebugEnabled()) {
             for (ManifestEntry entry : entries) {
                 LOG.debug("reading from " + entry.toString());
@@ -307,14 +313,16 @@ public class TestFileStore extends KeyValueFileStore {
                 RecordReaderIterator<KeyValue> iterator =
                         new RecordReaderIterator<>(
                                 read.createReader(
-                                        new Split(
+                                        new DataSplit(
+                                                0L /* unused */,
                                                 entryWithPartition.getKey(),
                                                 entryWithBucket.getKey(),
                                                 entryWithBucket.getValue(),
-                                                false)));
+                                                isIncremental)));
                 while (iterator.hasNext()) {
                     kvs.add(iterator.next().copy(keySerializer, valueSerializer));
                 }
+                iterator.close();
             }
         }
         return kvs;
@@ -374,27 +382,40 @@ public class TestFileStore extends KeyValueFileStore {
         if (actualFiles.remove(earliest)) {
             long earliestId = snapshotManager.readHint(SnapshotManager.EARLIEST);
             earliest.getFileSystem().delete(earliest, false);
-            assertThat(earliestId <= snapshotManager.findEarliest()).isTrue();
+            assertThat(earliestId <= snapshotManager.earliestSnapshotId()).isTrue();
         }
         if (actualFiles.remove(latest)) {
             long latestId = snapshotManager.readHint(SnapshotManager.LATEST);
             latest.getFileSystem().delete(latest, false);
-            assertThat(latestId <= snapshotManager.findLatest()).isTrue();
+            assertThat(latestId <= snapshotManager.latestSnapshotId()).isTrue();
         }
         actualFiles.remove(latest);
 
-        assertThat(actualFiles).isEqualTo(filesInUse);
+        // for easier debugging
+        String expectedString =
+                filesInUse.stream().map(Path::toString).sorted().collect(Collectors.joining(",\n"));
+        String actualString =
+                actualFiles.stream()
+                        .map(Path::toString)
+                        .sorted()
+                        .collect(Collectors.joining(",\n"));
+        assertThat(actualString).isEqualTo(expectedString);
     }
 
     private Set<Path> getFilesInUse() {
+        Set<Path> result = new HashSet<>();
         FileStorePathFactory pathFactory = pathFactory();
         ManifestList manifestList = manifestListFactory().create();
         FileStoreScan scan = newScan();
 
+        SchemaManager schemaManager = new SchemaManager(options.path());
+        schemaManager.listAllIds().forEach(id -> result.add(schemaManager.toSchemaPath(id)));
+
         SnapshotManager snapshotManager = snapshotManager();
         Long latestSnapshotId = snapshotManager.latestSnapshotId();
+
         if (latestSnapshotId == null) {
-            return Collections.emptySet();
+            return result;
         }
 
         long firstInUseSnapshotId = Snapshot.FIRST_SNAPSHOT_ID;
@@ -405,7 +426,6 @@ public class TestFileStore extends KeyValueFileStore {
             }
         }
 
-        Set<Path> result = new HashSet<>();
         for (long id = firstInUseSnapshotId; id <= latestSnapshotId; id++) {
             Path snapshotPath = snapshotManager.snapshotPath(id);
             Snapshot snapshot = Snapshot.fromPath(snapshotPath);
@@ -416,9 +436,15 @@ public class TestFileStore extends KeyValueFileStore {
             // manifest lists
             result.add(pathFactory.toManifestListPath(snapshot.baseManifestList()));
             result.add(pathFactory.toManifestListPath(snapshot.deltaManifestList()));
+            if (snapshot.changelogManifestList() != null) {
+                result.add(pathFactory.toManifestListPath(snapshot.changelogManifestList()));
+            }
 
             // manifests
-            List<ManifestFileMeta> manifests = snapshot.readAllManifests(manifestList);
+            List<ManifestFileMeta> manifests = snapshot.readAllDataManifests(manifestList);
+            if (snapshot.changelogManifestList() != null) {
+                manifests.addAll(manifestList.read(snapshot.changelogManifestList()));
+            }
             manifests.forEach(m -> result.add(pathFactory.toManifestFilePath(m.fileName())));
 
             // data file
@@ -432,4 +458,73 @@ public class TestFileStore extends KeyValueFileStore {
         }
         return result;
     }
+
+    /** Builder of {@link TestFileStore}. */
+    public static class Builder {
+
+        private final String format;
+        private final String root;
+        private final int numBuckets;
+        private final RowType partitionType;
+        private final RowType keyType;
+        private final RowType valueType;
+        private final KeyValueFieldsExtractor keyValueFieldsExtractor;
+        private final MergeFunctionFactory<KeyValue> mfFactory;
+
+        private CoreOptions.ChangelogProducer changelogProducer;
+
+        public Builder(
+                String format,
+                String root,
+                int numBuckets,
+                RowType partitionType,
+                RowType keyType,
+                RowType valueType,
+                KeyValueFieldsExtractor keyValueFieldsExtractor,
+                MergeFunctionFactory<KeyValue> mfFactory) {
+            this.format = format;
+            this.root = root;
+            this.numBuckets = numBuckets;
+            this.partitionType = partitionType;
+            this.keyType = keyType;
+            this.valueType = valueType;
+            this.keyValueFieldsExtractor = keyValueFieldsExtractor;
+            this.mfFactory = mfFactory;
+
+            this.changelogProducer = CoreOptions.ChangelogProducer.NONE;
+        }
+
+        public Builder changelogProducer(CoreOptions.ChangelogProducer changelogProducer) {
+            this.changelogProducer = changelogProducer;
+            return this;
+        }
+
+        public TestFileStore build() {
+            Configuration conf = new Configuration();
+
+            conf.set(CoreOptions.WRITE_BUFFER_SIZE, WRITE_BUFFER_SIZE);
+            conf.set(CoreOptions.PAGE_SIZE, PAGE_SIZE);
+            conf.set(CoreOptions.TARGET_FILE_SIZE, MemorySize.parse("1 kb"));
+
+            conf.set(
+                    CoreOptions.MANIFEST_TARGET_FILE_SIZE,
+                    MemorySize.parse((ThreadLocalRandom.current().nextInt(16) + 1) + "kb"));
+
+            conf.set(CoreOptions.FILE_FORMAT, format);
+            conf.set(CoreOptions.MANIFEST_FORMAT, format);
+            conf.set(CoreOptions.PATH, root);
+            conf.set(CoreOptions.BUCKET, numBuckets);
+
+            conf.set(CoreOptions.CHANGELOG_PRODUCER, changelogProducer);
+
+            return new TestFileStore(
+                    root,
+                    new CoreOptions(conf),
+                    partitionType,
+                    keyType,
+                    valueType,
+                    keyValueFieldsExtractor,
+                    mfFactory);
+        }
+    }
 }
