@@ -18,6 +18,7 @@
 
 package org.apache.flink.table.store.file;
 
+import org.apache.flink.core.fs.Path;
 import org.apache.flink.table.data.GenericArrayData;
 import org.apache.flink.table.data.GenericRowData;
 import org.apache.flink.table.data.RowData;
@@ -25,6 +26,11 @@ import org.apache.flink.table.data.StringData;
 import org.apache.flink.table.data.binary.BinaryRowData;
 import org.apache.flink.table.runtime.generated.RecordComparator;
 import org.apache.flink.table.runtime.typeutils.RowDataSerializer;
+import org.apache.flink.table.store.file.schema.DataField;
+import org.apache.flink.table.store.file.schema.KeyValueFieldsExtractor;
+import org.apache.flink.table.store.file.schema.SchemaManager;
+import org.apache.flink.table.store.file.schema.TableSchema;
+import org.apache.flink.table.store.table.SchemaEvolutionTableTestBase;
 import org.apache.flink.table.types.logical.ArrayType;
 import org.apache.flink.table.types.logical.BigIntType;
 import org.apache.flink.table.types.logical.IntType;
@@ -37,6 +43,7 @@ import javax.annotation.Nullable;
 
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -104,6 +111,8 @@ public class TestKeyValueGenerator {
                     new String[] {"shopId", "orderId", "itemId", "priceAmount", "comment"});
     public static final RowType NON_PARTITIONED_PART_TYPE = RowType.of();
 
+    public static final List<String> KEY_NAME_LIST = Arrays.asList("shopId", "orderId");
+
     public static final RowType KEY_TYPE =
             RowType.of(
                     new LogicalType[] {new IntType(false), new BigIntType(false)},
@@ -264,6 +273,21 @@ public class TestKeyValueGenerator {
         return map;
     }
 
+    public static SchemaManager createTestSchemaManager(Path path) {
+        TableSchema tableSchema =
+                new TableSchema(
+                        0,
+                        TableSchema.newFields(DEFAULT_ROW_TYPE),
+                        DEFAULT_ROW_TYPE.getFieldCount(),
+                        Collections.EMPTY_LIST,
+                        KEY_NAME_LIST,
+                        Collections.EMPTY_MAP,
+                        "");
+        Map<Long, TableSchema> schemas = new HashMap<>();
+        schemas.put(tableSchema.id(), tableSchema);
+        return new SchemaEvolutionTableTestBase.TestingSchemaManager(path, schemas);
+    }
+
     public void sort(List<KeyValue> kvs) {
         kvs.sort(
                 (a, b) -> {
@@ -323,4 +347,25 @@ public class TestKeyValueGenerator {
         SINGLE_PARTITIONED,
         MULTI_PARTITIONED
     }
+
+    /** {@link KeyValueFieldsExtractor} implementation for test. */
+    public static class TestKeyValueFieldsExtractor implements KeyValueFieldsExtractor {
+        private static final long serialVersionUID = 1L;
+
+        public static final TestKeyValueFieldsExtractor EXTRACTOR =
+                new TestKeyValueFieldsExtractor();
+
+        @Override
+        public List<DataField> keyFields(TableSchema schema) {
+            return schema.fields().stream()
+                    .filter(f -> KEY_NAME_LIST.contains(f.name()))
+                    .map(f -> new DataField(f.id(), "key_" + f.name(), f.type(), f.description()))
+                    .collect(Collectors.toList());
+        }
+
+        @Override
+        public List<DataField> valueFields(TableSchema schema) {
+            return schema.fields();
+        }
+    }
 }
