@@ -24,27 +24,34 @@ import org.apache.flink.core.fs.FileStatus;
 import org.apache.flink.core.fs.Path;
 import org.apache.flink.table.data.GenericRowData;
 import org.apache.flink.table.data.RowData;
-import org.apache.flink.table.data.binary.BinaryRowDataUtil;
 import org.apache.flink.table.store.CoreOptions;
 import org.apache.flink.table.store.CoreOptions.ChangelogProducer;
 import org.apache.flink.table.store.file.KeyValue;
-import org.apache.flink.table.store.file.data.DataFileMeta;
-import org.apache.flink.table.store.file.data.DataFileReader;
-import org.apache.flink.table.store.file.data.DataFileWriter;
+import org.apache.flink.table.store.file.compact.CompactResult;
 import org.apache.flink.table.store.file.format.FlushingFileFormat;
+import org.apache.flink.table.store.file.io.DataFileMeta;
+import org.apache.flink.table.store.file.io.KeyValueFileReaderFactory;
+import org.apache.flink.table.store.file.io.KeyValueFileWriterFactory;
+import org.apache.flink.table.store.file.io.RollingFileWriter;
 import org.apache.flink.table.store.file.memory.HeapMemorySegmentPool;
-import org.apache.flink.table.store.file.mergetree.compact.CompactRewriter;
+import org.apache.flink.table.store.file.mergetree.compact.AbstractCompactRewriter;
 import org.apache.flink.table.store.file.mergetree.compact.CompactStrategy;
 import org.apache.flink.table.store.file.mergetree.compact.DeduplicateMergeFunction;
 import org.apache.flink.table.store.file.mergetree.compact.IntervalPartition;
 import org.apache.flink.table.store.file.mergetree.compact.MergeTreeCompactManager;
 import org.apache.flink.table.store.file.mergetree.compact.UniversalCompaction;
+import org.apache.flink.table.store.file.schema.AtomicDataType;
+import org.apache.flink.table.store.file.schema.DataField;
+import org.apache.flink.table.store.file.schema.KeyValueFieldsExtractor;
 import org.apache.flink.table.store.file.schema.SchemaManager;
+import org.apache.flink.table.store.file.schema.TableSchema;
 import org.apache.flink.table.store.file.utils.FileStorePathFactory;
 import org.apache.flink.table.store.file.utils.RecordReader;
 import org.apache.flink.table.store.file.utils.RecordReaderIterator;
-import org.apache.flink.table.store.file.writer.RecordWriter;
+import org.apache.flink.table.store.file.utils.RecordWriter;
 import org.apache.flink.table.store.format.FileFormat;
+import org.apache.flink.table.store.table.SchemaEvolutionTableTestBase;
+import org.apache.flink.table.store.utils.BinaryRowDataUtil;
 import org.apache.flink.table.types.logical.IntType;
 import org.apache.flink.table.types.logical.RowType;
 import org.apache.flink.types.RowKind;
@@ -62,8 +69,10 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Comparator;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Random;
 import java.util.Set;
 import java.util.TreeMap;
@@ -74,7 +83,7 @@ import java.util.stream.Collectors;
 import static java.util.Collections.singletonList;
 import static org.assertj.core.api.Assertions.assertThat;
 
-/** Tests for {@link MergeTreeReader} and {@link MergeTreeWriter}. */
+/** Tests for {@link MergeTreeReaders} and {@link MergeTreeWriter}. */
 public class MergeTreeTest {
 
     @TempDir java.nio.file.Path tempDir;
@@ -84,8 +93,10 @@ public class MergeTreeTest {
     private Comparator<RowData> comparator;
 
     private CoreOptions options;
-    private DataFileReader dataFileReader;
-    private DataFileWriter dataFileWriter;
+    private KeyValueFileReaderFactory readerFactory;
+    private KeyValueFileReaderFactory compactReaderFactory;
+    private KeyValueFileWriterFactory writerFactory;
+    private KeyValueFileWriterFactory compactWriterFactory;
     private RecordWriter<KeyValue> writer;
 
     @BeforeEach
@@ -94,10 +105,26 @@ public class MergeTreeTest {
         pathFactory = new FileStorePathFactory(path);
         comparator = Comparator.comparingInt(o -> o.getInt(0));
         recreateMergeTree(1024 * 1024);
-        Path bucketDir = dataFileWriter.pathFactory().toPath("ignore").getParent();
+        Path bucketDir = writerFactory.pathFactory().toPath("ignore").getParent();
         bucketDir.getFileSystem().mkdirs(bucketDir);
     }
 
+    private SchemaManager createTestingSchemaManager(Path path) {
+        TableSchema schema =
+                new TableSchema(
+                        0,
+                        new ArrayList<>(),
+                        -1,
+                        new ArrayList<>(),
+                        new ArrayList<>(),
+                        new HashMap<>(),
+                        "");
+        Map<Long, TableSchema> schemas = new HashMap<>();
+        schemas.put(schema.id(), schema);
+
+        return new SchemaEvolutionTableTestBase.TestingSchemaManager(path, schemas);
+    }
+
     private void recreateMergeTree(long targetFileSize) {
         Configuration configuration = new Configuration();
         configuration.set(CoreOptions.WRITE_BUFFER_SIZE, new MemorySize(4096 * 3));
@@ -106,25 +133,38 @@ public class MergeTreeTest {
         options = new CoreOptions(configuration);
         RowType keyType = new RowType(singletonList(new RowType.RowField("k", new IntType())));
         RowType valueType = new RowType(singletonList(new RowType.RowField("v", new IntType())));
+
         FileFormat flushingAvro = new FlushingFileFormat("avro");
-        dataFileReader =
-                new DataFileReader.Factory(
-                                new SchemaManager(path),
-                                0,
-                                keyType,
-                                valueType,
-                                flushingAvro,
-                                pathFactory)
-                        .create(BinaryRowDataUtil.EMPTY_ROW, 0);
-        dataFileWriter =
-                new DataFileWriter.Factory(
-                                0,
-                                keyType,
-                                valueType,
-                                flushingAvro,
-                                pathFactory,
-                                options.targetFileSize())
-                        .create(BinaryRowDataUtil.EMPTY_ROW, 0);
+        KeyValueFileReaderFactory.Builder readerFactoryBuilder =
+                KeyValueFileReaderFactory.builder(
+                        createTestingSchemaManager(path),
+                        0,
+                        keyType,
+                        valueType,
+                        flushingAvro,
+                        pathFactory,
+                        new KeyValueFieldsExtractor() {
+                            @Override
+                            public List<DataField> keyFields(TableSchema schema) {
+                                return Collections.singletonList(
+                                        new DataField(
+                                                0, "k", new AtomicDataType(new IntType(false))));
+                            }
+
+                            @Override
+                            public List<DataField> valueFields(TableSchema schema) {
+                                return Collections.singletonList(
+                                        new DataField(
+                                                0, "v", new AtomicDataType(new IntType(false))));
+                            }
+                        });
+        readerFactory = readerFactoryBuilder.build(BinaryRowDataUtil.EMPTY_ROW, 0);
+        compactReaderFactory = readerFactoryBuilder.build(BinaryRowDataUtil.EMPTY_ROW, 0);
+        KeyValueFileWriterFactory.Builder writerFactoryBuilder =
+                KeyValueFileWriterFactory.builder(
+                        0, keyType, valueType, flushingAvro, pathFactory, options.targetFileSize());
+        writerFactory = writerFactoryBuilder.build(BinaryRowDataUtil.EMPTY_ROW, 0);
+        compactWriterFactory = writerFactoryBuilder.build(BinaryRowDataUtil.EMPTY_ROW, 0);
         writer = createMergeTreeWriter(Collections.emptyList());
     }
 
@@ -167,7 +207,7 @@ public class MergeTreeTest {
     @Test
     public void testRestore() throws Exception {
         List<TestRecord> expected = new ArrayList<>(writeBatch());
-        List<DataFileMeta> newFiles = writer.prepareCommit(true).newFiles();
+        List<DataFileMeta> newFiles = writer.prepareCommit(true).newFilesIncrement().newFiles();
         writer = createMergeTreeWriter(newFiles);
         expected.addAll(writeBatch());
         writer.prepareCommit(true);
@@ -175,16 +215,6 @@ public class MergeTreeTest {
         assertRecords(expected);
     }
 
-    @Test
-    public void testClose() throws Exception {
-        doTestWriteRead(6);
-        List<DataFileMeta> files = writer.close();
-        for (DataFileMeta file : files) {
-            Path path = dataFileWriter.pathFactory().toPath(file.fileName());
-            assertThat(path.getFileSystem().exists(path)).isFalse();
-        }
-    }
-
     @ParameterizedTest
     @ValueSource(longs = {1, 1024 * 1024})
     public void testCloseUpgrade(long targetFileSize) throws Exception {
@@ -207,7 +237,7 @@ public class MergeTreeTest {
             }
             writeAll(records);
             expected.addAll(records);
-            Increment increment = writer.prepareCommit(true);
+            RecordWriter.CommitIncrement increment = writer.prepareCommit(true);
             mergeCompacted(newFileNames, compactedFiles, increment);
         }
         writer.close();
@@ -238,8 +268,8 @@ public class MergeTreeTest {
                 writer.sync();
             }
 
-            Increment increment = writer.prepareCommit(true);
-            newFiles.addAll(increment.newFiles());
+            RecordWriter.CommitIncrement increment = writer.prepareCommit(true);
+            newFiles.addAll(increment.newFilesIncrement().newFiles());
             mergeCompacted(newFileNames, compactedFiles, increment);
         }
 
@@ -253,7 +283,9 @@ public class MergeTreeTest {
         // assert records from increment compacted files
         assertRecords(expected, compactedFiles, true);
 
-        Path bucketDir = dataFileWriter.pathFactory().toPath("ignore").getParent();
+        writer.close();
+
+        Path bucketDir = writerFactory.pathFactory().toPath("ignore").getParent();
         Set<String> files =
                 Arrays.stream(bucketDir.getFileSystem().listStatus(bucketDir))
                         .map(FileStatus::getPath)
@@ -269,16 +301,15 @@ public class MergeTreeTest {
                 files.stream().map(DataFileMeta::maxSequenceNumber).max(Long::compare).orElse(-1L);
         MergeTreeWriter writer =
                 new MergeTreeWriter(
-                        dataFileWriter.keyType(),
-                        dataFileWriter.valueType(),
-                        createCompactManager(dataFileWriter, service, files),
-                        new Levels(comparator, files, options.numLevels()),
+                        false,
+                        128,
+                        null,
+                        createCompactManager(service, files),
                         maxSequenceNumber,
                         comparator,
-                        new DeduplicateMergeFunction(),
-                        dataFileWriter,
+                        DeduplicateMergeFunction.factory().create(),
+                        writerFactory,
                         options.commitForceCompact(),
-                        options.numSortedRunStopTrigger(),
                         ChangelogProducer.NONE);
         writer.setMemoryPool(
                 new HeapMemorySegmentPool(options.writeBufferSize(), options.pageSize()));
@@ -286,52 +317,44 @@ public class MergeTreeTest {
     }
 
     private MergeTreeCompactManager createCompactManager(
-            DataFileWriter dataFileWriter,
-            ExecutorService compactExecutor,
-            List<DataFileMeta> files) {
+            ExecutorService compactExecutor, List<DataFileMeta> files) {
         CompactStrategy strategy =
                 new UniversalCompaction(
                         options.maxSizeAmplificationPercent(),
                         options.sortedRunSizeRatio(),
                         options.numSortedRunCompactionTrigger(),
                         options.maxSortedRunNum());
-        CompactRewriter rewriter =
-                (outputLevel, dropDelete, sections) ->
-                        dataFileWriter.write(
-                                new RecordReaderIterator<>(
-                                        new MergeTreeReader(
-                                                sections,
-                                                dropDelete,
-                                                dataFileReader,
-                                                comparator,
-                                                new DeduplicateMergeFunction())),
-                                outputLevel);
         return new MergeTreeCompactManager(
                 compactExecutor,
                 new Levels(comparator, files, options.numLevels()),
                 strategy,
                 comparator,
                 options.targetFileSize(),
-                rewriter);
+                options.numSortedRunStopTrigger(),
+                new TestRewriter());
     }
 
     private void mergeCompacted(
-            Set<String> newFileNames, List<DataFileMeta> compactedFiles, Increment increment) {
-        increment.newFiles().stream().map(DataFileMeta::fileName).forEach(newFileNames::add);
-        compactedFiles.addAll(increment.newFiles());
+            Set<String> newFileNames,
+            List<DataFileMeta> compactedFiles,
+            RecordWriter.CommitIncrement increment) {
+        increment.newFilesIncrement().newFiles().stream()
+                .map(DataFileMeta::fileName)
+                .forEach(newFileNames::add);
+        compactedFiles.addAll(increment.newFilesIncrement().newFiles());
         Set<String> afterFiles =
-                increment.compactAfter().stream()
+                increment.compactIncrement().compactAfter().stream()
                         .map(DataFileMeta::fileName)
                         .collect(Collectors.toSet());
-        for (DataFileMeta file : increment.compactBefore()) {
+        for (DataFileMeta file : increment.compactIncrement().compactBefore()) {
             boolean remove = compactedFiles.remove(file);
             assertThat(remove).isTrue();
             // See MergeTreeWriter.updateCompactResult
             if (!newFileNames.contains(file.fileName()) && !afterFiles.contains(file.fileName())) {
-                dataFileWriter.delete(file);
+                compactWriterFactory.deleteFile(file.fileName());
             }
         }
-        compactedFiles.addAll(increment.compactAfter());
+        compactedFiles.addAll(increment.compactIncrement().compactAfter());
     }
 
     private List<TestRecord> writeBatch() throws Exception {
@@ -346,7 +369,10 @@ public class MergeTreeTest {
 
     private void assertRecords(List<TestRecord> expected) throws Exception {
         // compaction will drop delete
-        List<DataFileMeta> files = ((MergeTreeWriter) writer).levels().allFiles();
+        List<DataFileMeta> files =
+                ((MergeTreeCompactManager) ((MergeTreeWriter) writer).compactManager())
+                        .levels()
+                        .allFiles();
         assertRecords(expected, files, true);
     }
 
@@ -379,14 +405,14 @@ public class MergeTreeTest {
     private List<TestRecord> readAll(List<DataFileMeta> files, boolean dropDelete)
             throws Exception {
         RecordReader<KeyValue> reader =
-                new MergeTreeReader(
+                MergeTreeReaders.readerForMergeTree(
                         new IntervalPartition(files, comparator).partition(),
                         dropDelete,
-                        dataFileReader,
+                        readerFactory,
                         comparator,
-                        new DeduplicateMergeFunction());
+                        DeduplicateMergeFunction.factory().create());
         List<TestRecord> records = new ArrayList<>();
-        try (RecordReaderIterator<KeyValue> iterator = new RecordReaderIterator<KeyValue>(reader)) {
+        try (RecordReaderIterator<KeyValue> iterator = new RecordReaderIterator<>(reader)) {
             while (iterator.hasNext()) {
                 KeyValue kv = iterator.next();
                 records.add(
@@ -413,6 +439,27 @@ public class MergeTreeTest {
         return records;
     }
 
+    private class TestRewriter extends AbstractCompactRewriter {
+
+        @Override
+        public CompactResult rewrite(
+                int outputLevel, boolean dropDelete, List<List<SortedRun>> sections)
+                throws Exception {
+            RollingFileWriter<KeyValue, DataFileMeta> writer =
+                    writerFactory.createRollingMergeTreeFileWriter(outputLevel);
+            RecordReader<KeyValue> sectionsReader =
+                    MergeTreeReaders.readerForMergeTree(
+                            sections,
+                            dropDelete,
+                            compactReaderFactory,
+                            comparator,
+                            DeduplicateMergeFunction.factory().create());
+            writer.write(new RecordReaderIterator<>(sectionsReader));
+            writer.close();
+            return new CompactResult(extractFilesFromSections(sections), writer.result());
+        }
+    }
+
     private static class TestRecord {
 
         private final RowKind kind;
