@@ -32,11 +32,14 @@ import java.util.List;
 /** Tests for {@link SortMergeReader}. */
 public abstract class SortMergeReaderTestBase extends CombiningRecordReaderTestBase {
 
-    protected abstract MergeFunction createMergeFunction();
+    protected abstract MergeFunction<KeyValue> createMergeFunction();
 
     @Override
     protected RecordReader<KeyValue> createRecordReader(List<TestReusingRecordReader> readers) {
-        return new SortMergeReader(new ArrayList<>(readers), KEY_COMPARATOR, createMergeFunction());
+        return new SortMergeReader<>(
+                new ArrayList<>(readers),
+                KEY_COMPARATOR,
+                new ReducerMergeFunctionWrapper(createMergeFunction()));
     }
 
     @Test
@@ -85,7 +88,7 @@ public abstract class SortMergeReaderTestBase extends CombiningRecordReaderTestB
         }
 
         @Override
-        protected MergeFunction createMergeFunction() {
+        protected MergeFunction<KeyValue> createMergeFunction() {
             return new DeduplicateMergeFunction();
         }
     }
@@ -104,7 +107,7 @@ public abstract class SortMergeReaderTestBase extends CombiningRecordReaderTestB
         }
 
         @Override
-        protected MergeFunction createMergeFunction() {
+        protected MergeFunction<KeyValue> createMergeFunction() {
             return new ValueCountMergeFunction();
         }
 
