@@ -19,13 +19,14 @@
 package org.apache.flink.table.store.file.mergetree.compact;
 
 import org.apache.flink.table.store.file.compact.CompactUnit;
-import org.apache.flink.table.store.file.data.DataFileMeta;
+import org.apache.flink.table.store.file.io.DataFileMeta;
 import org.apache.flink.table.store.file.mergetree.LevelSortedRun;
 import org.apache.flink.table.store.file.mergetree.SortedRun;
 
 import org.junit.jupiter.api.Test;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
 import java.util.Optional;
 import java.util.stream.Collectors;
@@ -39,9 +40,9 @@ public class UniversalCompactionTest {
     @Test
     public void testOutputLevel() {
         assertThat(createUnit(createLevels(0, 0, 1, 3, 4), 5, 1, Integer.MAX_VALUE).outputLevel())
-                .isEqualTo(0);
+                .isEqualTo(1);
         assertThat(createUnit(createLevels(0, 0, 1, 3, 4), 5, 2, Integer.MAX_VALUE).outputLevel())
-                .isEqualTo(0);
+                .isEqualTo(1);
         assertThat(createUnit(createLevels(0, 0, 1, 3, 4), 5, 3, Integer.MAX_VALUE).outputLevel())
                 .isEqualTo(2);
         assertThat(createUnit(createLevels(0, 0, 1, 3, 4), 5, 4, Integer.MAX_VALUE).outputLevel())
@@ -61,13 +62,17 @@ public class UniversalCompactionTest {
         assertThat(results).isEqualTo(new long[] {1, 2, 3, 3});
 
         // by size ratio
-        pick = compaction.pick(3, level0(1, 1, 1, 50));
+        pick =
+                compaction.pick(
+                        4, Arrays.asList(level(0, 1), level(1, 1), level(2, 1), level(3, 50)));
         assertThat(pick.isPresent()).isTrue();
         results = pick.get().files().stream().mapToLong(DataFileMeta::fileSize).toArray();
         assertThat(results).isEqualTo(new long[] {1, 1, 1});
 
         // by file num
-        pick = compaction.pick(3, level0(1, 2, 3, 50));
+        pick =
+                compaction.pick(
+                        4, Arrays.asList(level(0, 1), level(1, 2), level(2, 3), level(3, 50)));
         assertThat(pick.isPresent()).isTrue();
         results = pick.get().files().stream().mapToLong(DataFileMeta::fileSize).toArray();
         // 3 should be in the candidate, by size ratio after picking by file num
@@ -85,19 +90,43 @@ public class UniversalCompactionTest {
         assertThat(results).isEqualTo(new long[] {1, 2, 3, 3});
 
         // by size ratio
-        pick = compaction.pick(3, level0(1, 1, 1, 50));
+        pick =
+                compaction.pick(
+                        4, Arrays.asList(level(0, 1), level(1, 1), level(2, 1), level(3, 50)));
         assertThat(pick.isPresent()).isTrue();
         results = pick.get().files().stream().mapToLong(DataFileMeta::fileSize).toArray();
         assertThat(results).isEqualTo(new long[] {1, 1});
 
         // by file num
-        pick = compaction.pick(3, level0(1, 2, 3, 50));
+        pick =
+                compaction.pick(
+                        4, Arrays.asList(level(0, 1), level(1, 2), level(2, 3), level(3, 50)));
         assertThat(pick.isPresent()).isTrue();
         results = pick.get().files().stream().mapToLong(DataFileMeta::fileSize).toArray();
         // 3 should be in the candidate, by size ratio after picking by file num
         assertThat(results).isEqualTo(new long[] {1, 2});
     }
 
+    @Test
+    public void testNoOutputLevel0() {
+        UniversalCompaction compaction = new UniversalCompaction(25, 1, 3, 2);
+
+        Optional<CompactUnit> pick =
+                compaction.pick(
+                        3, Arrays.asList(level(0, 1), level(0, 1), level(1, 1), level(2, 50)));
+        assertThat(pick.isPresent()).isTrue();
+        long[] results = pick.get().files().stream().mapToLong(DataFileMeta::fileSize).toArray();
+        assertThat(results).isEqualTo(new long[] {1, 1, 1});
+
+        pick =
+                compaction.pick(
+                        3, Arrays.asList(level(0, 1), level(0, 2), level(1, 3), level(2, 50)));
+        assertThat(pick.isPresent()).isTrue();
+        results = pick.get().files().stream().mapToLong(DataFileMeta::fileSize).toArray();
+        // 3 should be in the candidate, by size ratio after picking by file num
+        assertThat(results).isEqualTo(new long[] {1, 2, 3});
+    }
+
     @Test
     public void testSizeAmplification() {
         UniversalCompaction compaction = new UniversalCompaction(25, 0, 1, Integer.MAX_VALUE);
@@ -226,7 +255,11 @@ public class UniversalCompactionTest {
     }
 
     private long[] pickForSizeRatio(UniversalCompaction compaction, long... sizes) {
-        CompactUnit unit = compaction.pickForSizeRatio(3, level0(sizes));
+        List<LevelSortedRun> runs = new ArrayList<>();
+        for (int i = 0; i < sizes.length; i++) {
+            runs.add(level(i, sizes[i]));
+        }
+        CompactUnit unit = compaction.pickForSizeRatio(sizes.length, runs);
         if (unit != null) {
             List<Long> compact =
                     unit.files().stream().map(DataFileMeta::fileSize).collect(Collectors.toList());
@@ -256,6 +289,10 @@ public class UniversalCompactionTest {
         return runs;
     }
 
+    private LevelSortedRun level(int level, long size) {
+        return new LevelSortedRun(level, SortedRun.fromSingle(file(size)));
+    }
+
     private DataFileMeta file(long size) {
         return new DataFileMeta("", size, 1, null, null, null, null, 0, 0, 0, 0);
     }
