@@ -25,6 +25,8 @@ import org.apache.flink.table.store.file.TestKeyValueGenerator;
 import org.apache.flink.table.store.file.manifest.ManifestCommittable;
 import org.apache.flink.table.store.file.memory.HeapMemorySegmentPool;
 import org.apache.flink.table.store.file.mergetree.MergeTreeWriter;
+import org.apache.flink.table.store.file.utils.RecordWriter;
+import org.apache.flink.table.store.table.sink.FileCommittable;
 import org.apache.flink.table.types.logical.RowType;
 
 import org.slf4j.Logger;
@@ -34,9 +36,10 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
-import java.util.Random;
+import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
@@ -58,10 +61,13 @@ public class TestCommitThread extends Thread {
     private final Map<BinaryRowData, List<KeyValue>> data;
     private final Map<BinaryRowData, List<KeyValue>> result;
     private final Map<BinaryRowData, MergeTreeWriter> writers;
+    private final Set<BinaryRowData> writtenPartitions;
 
-    private final FileStoreWrite<KeyValue> write;
+    private final AbstractFileStoreWrite<KeyValue> write;
     private final FileStoreCommit commit;
 
+    private long commitIdentifier;
+
     public TestCommitThread(
             RowType keyType,
             RowType valueType,
@@ -75,6 +81,7 @@ public class TestCommitThread extends Thread {
         this.data = data;
         this.result = new HashMap<>();
         this.writers = new HashMap<>();
+        this.writtenPartitions = new HashSet<>();
 
         if (LOG.isDebugEnabled()) {
             LOG.debug(
@@ -85,8 +92,11 @@ public class TestCommitThread extends Thread {
                                     .collect(Collectors.joining("\n")));
         }
 
-        this.write = safeStore.newWrite();
-        this.commit = testStore.newCommit(UUID.randomUUID().toString());
+        String commitUser = UUID.randomUUID().toString();
+        this.write = safeStore.newWrite(commitUser);
+        this.commit = testStore.newCommit(commitUser).withCreateEmptyCommit(true);
+
+        this.commitIdentifier = 0;
     }
 
     public List<KeyValue> getResult() {
@@ -107,6 +117,7 @@ public class TestCommitThread extends Thread {
                 throw new RuntimeException(e);
             }
         }
+        doFinalCompact();
 
         for (MergeTreeWriter writer : writers.values()) {
             try {
@@ -122,10 +133,12 @@ public class TestCommitThread extends Thread {
         for (int i = 0; i < numWrites && !data.isEmpty(); i++) {
             writeData();
         }
-        ManifestCommittable committable =
-                new ManifestCommittable(String.valueOf(new Random().nextLong()));
+        ManifestCommittable committable = new ManifestCommittable(commitIdentifier++);
         for (Map.Entry<BinaryRowData, MergeTreeWriter> entry : writers.entrySet()) {
-            committable.addFileCommittable(entry.getKey(), 0, entry.getValue().prepareCommit(true));
+            RecordWriter.CommitIncrement inc = entry.getValue().prepareCommit(true);
+            committable.addFileCommittable(
+                    new FileCommittable(
+                            entry.getKey(), 0, inc.newFilesIncrement(), inc.compactIncrement()));
         }
 
         runWithRetry(committable, () -> commit.commit(committable, Collections.emptyMap()));
@@ -133,9 +146,10 @@ public class TestCommitThread extends Thread {
 
     private void doOverwrite() throws Exception {
         BinaryRowData partition = overwriteData();
-        ManifestCommittable committable =
-                new ManifestCommittable(String.valueOf(new Random().nextLong()));
-        committable.addFileCommittable(partition, 0, writers.get(partition).prepareCommit(true));
+        ManifestCommittable committable = new ManifestCommittable(commitIdentifier++);
+        RecordWriter.CommitIncrement inc = writers.get(partition).prepareCommit(true);
+        committable.addFileCommittable(
+                new FileCommittable(partition, 0, inc.newFilesIncrement(), inc.compactIncrement()));
 
         runWithRetry(
                 committable,
@@ -146,6 +160,31 @@ public class TestCommitThread extends Thread {
                                 Collections.emptyMap()));
     }
 
+    private void doFinalCompact() {
+        long identifier = commitIdentifier++;
+        while (true) {
+            try {
+                ManifestCommittable committable = new ManifestCommittable(identifier);
+                for (BinaryRowData partition : writtenPartitions) {
+                    MergeTreeWriter writer =
+                            writers.computeIfAbsent(partition, p -> createWriter(p, false));
+                    writer.compact(true);
+                    RecordWriter.CommitIncrement inc = writer.prepareCommit(true);
+                    committable.addFileCommittable(
+                            new FileCommittable(
+                                    partition, 0, inc.newFilesIncrement(), inc.compactIncrement()));
+                }
+                commit.commit(committable, Collections.emptyMap());
+                break;
+            } catch (Exception e) {
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Failed to do final compact because of exception, try again", e);
+                }
+                writers.clear();
+            }
+        }
+    }
+
     private void runWithRetry(ManifestCommittable committable, Runnable runnable) {
         boolean shouldCheckFilter = false;
         while (true) {
@@ -171,6 +210,7 @@ public class TestCommitThread extends Thread {
         List<KeyValue> changes = new ArrayList<>();
         BinaryRowData partition = pickData(changes);
         result.computeIfAbsent(partition, p -> new ArrayList<>()).addAll(changes);
+        writtenPartitions.add(partition);
 
         if (LOG.isDebugEnabled()) {
             LOG.debug(
@@ -239,8 +279,10 @@ public class TestCommitThread extends Thread {
                         });
         MergeTreeWriter writer =
                 empty
-                        ? (MergeTreeWriter) write.createEmptyWriter(partition, 0, service)
-                        : (MergeTreeWriter) write.createWriter(partition, 0, service);
+                        ? (MergeTreeWriter)
+                                write.createEmptyWriterContainer(partition, 0, service).writer
+                        : (MergeTreeWriter)
+                                write.createWriterContainer(partition, 0, service).writer;
         writer.setMemoryPool(
                 new HeapMemorySegmentPool(
                         WRITE_BUFFER_SIZE.getBytes(), (int) PAGE_SIZE.getBytes()));
