@@ -30,6 +30,7 @@ import org.testcontainers.containers.output.Slf4jLogConsumer;
 import org.testcontainers.containers.wait.strategy.Wait;
 
 import java.io.File;
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
@@ -51,14 +52,20 @@ public abstract class E2eTestBase {
 
     private final boolean withKafka;
     private final boolean withHive;
+    private final boolean withSpark;
 
     protected E2eTestBase() {
         this(false, false);
     }
 
     protected E2eTestBase(boolean withKafka, boolean withHive) {
+        this(withKafka, withHive, false);
+    }
+
+    protected E2eTestBase(boolean withKafka, boolean withHive, boolean withSpark) {
         this.withKafka = withKafka;
         this.withHive = withHive;
+        this.withSpark = withSpark;
     }
 
     protected static final String TEST_DATA_DIR = "/test-data";
@@ -70,7 +77,7 @@ public abstract class E2eTestBase {
     private final List<String> currentResults = new ArrayList<>();
 
     protected DockerComposeContainer<?> environment;
-    private ContainerState jobManager;
+    protected ContainerState jobManager;
 
     @BeforeEach
     public void before() throws Exception {
@@ -108,6 +115,17 @@ public abstract class E2eTestBase {
             environment.waitingFor(
                     "hive-server_1", Wait.forLogMessage(".*Starting HiveServer2.*", 1));
         }
+        if (withSpark) {
+            List<String> sparkServices = Arrays.asList("spark-master", "spark-worker");
+            services.addAll(sparkServices);
+            for (String s : sparkServices) {
+                environment.withLogConsumer(s + "_1", new Slf4jLogConsumer(LOG));
+            }
+            environment.waitingFor(
+                    "spark-master_1",
+                    Wait.forLogMessage(
+                            ".*Master: I have been elected leader! New state: ALIVE.*", 1));
+        }
         environment.withServices(services.toArray(new String[0])).withLocalCompose(true);
 
         environment.start();
@@ -146,8 +164,40 @@ public abstract class E2eTestBase {
                 "cat >" + TEST_DATA_DIR + "/" + filename + " <<EOF\n" + content + "EOF\n");
     }
 
+    protected void createKafkaTopic(String topicName, int partitionNum)
+            throws IOException, InterruptedException {
+        assert withKafka;
+        ContainerState kafka = environment.getContainerByServiceName("kafka_1").get();
+        kafka.execInContainer(
+                "bash",
+                "-c",
+                String.format(
+                        "kafka-topics --create --bootstrap-server kafka:29092 --replication-factor 1 --partitions %d --topic %s",
+                        partitionNum, topicName));
+    }
+
+    protected void sendKafkaMessage(String filename, String content, String topicName)
+            throws IOException, InterruptedException {
+        assert withKafka;
+        ContainerState kafka = environment.getContainerByServiceName("kafka_1").get();
+        String uuid = topicName.substring("ts-topic-".length() + 1);
+        String tmpDir = "/tmp" + TEST_DATA_DIR + "/" + uuid;
+        kafka.execInContainer("mkdir", "-p", tmpDir);
+        kafka.execInContainer("touch", tmpDir + "/" + filename);
+        kafka.execInContainer(
+                "bash",
+                "-c",
+                String.format("cat > %s/%s <<EOF\n%s\nEOF\n", tmpDir, filename, content));
+        kafka.execInContainer(
+                "bash",
+                "-c",
+                String.format(
+                        "kafka-console-producer --bootstrap-server kafka:29092 --topic %s < %s/%s",
+                        topicName, tmpDir, filename));
+    }
+
     protected void runSql(String sql) throws Exception {
-        String fileName = UUID.randomUUID().toString() + ".sql";
+        String fileName = UUID.randomUUID() + ".sql";
         writeSharedFile(fileName, sql);
         Container.ExecResult execResult =
                 jobManager.execInContainer(
@@ -164,7 +214,7 @@ public abstract class E2eTestBase {
 
     protected String createResultSink(String sinkName, String schema) {
         String testDataSinkDdl =
-                "CREATE TABLE %s ( %s ) WITH (\n"
+                "CREATE TEMPORARY TABLE %s ( %s ) WITH (\n"
                         + "    'connector' = 'print',\n"
                         + "    'print-identifier' = '%s'\n"
                         + ");";
