@@ -76,6 +76,9 @@ public class TableStoreHiveStorageHandlerITCase {
 
     private static String engine;
 
+    private String commitUser;
+    private long commitIdentifier;
+
     @BeforeClass
     public static void beforeClass() {
         // TODO Currently FlinkEmbeddedHiveRunner can only be used for one test class,
@@ -98,6 +101,9 @@ public class TableStoreHiveStorageHandlerITCase {
 
         hiveShell.execute("CREATE DATABASE IF NOT EXISTS test_db");
         hiveShell.execute("USE test_db");
+
+        commitUser = UUID.randomUUID().toString();
+        commitIdentifier = 0;
     }
 
     @After
@@ -621,15 +627,17 @@ public class TableStoreHiveStorageHandlerITCase {
 
     private String writeData(FileStoreTable table, String path, List<RowData> data)
             throws Exception {
-        TableWrite write = table.newWrite();
-        TableCommit commit = table.newCommit("user");
+        TableWrite write = table.newWrite(commitUser);
+        TableCommit commit = table.newCommit(commitUser);
         for (RowData rowData : data) {
             write.write(rowData);
             if (ThreadLocalRandom.current().nextInt(5) == 0) {
-                commit.commit(UUID.randomUUID().toString(), write.prepareCommit(false));
+                commit.commit(commitIdentifier, write.prepareCommit(false, commitIdentifier));
+                commitIdentifier++;
             }
         }
-        commit.commit(UUID.randomUUID().toString(), write.prepareCommit(true));
+        commit.commit(commitIdentifier, write.prepareCommit(true, commitIdentifier));
+        commitIdentifier++;
         write.close();
 
         String tableName = "test_table_" + (UUID.randomUUID().toString().substring(0, 4));
@@ -669,12 +677,12 @@ public class TableStoreHiveStorageHandlerITCase {
             }
         }
 
-        TableWrite write = table.newWrite();
-        TableCommit commit = table.newCommit("user");
+        TableWrite write = table.newWrite(commitUser);
+        TableCommit commit = table.newCommit(commitUser);
         for (GenericRowData rowData : input) {
             write.write(rowData);
         }
-        commit.commit("0", write.prepareCommit(true));
+        commit.commit(0, write.prepareCommit(true, 0));
         write.close();
 
         hiveShell.execute(
@@ -776,20 +784,20 @@ public class TableStoreHiveStorageHandlerITCase {
 
         // TODO add NaN related tests after FLINK-27627 and FLINK-27628 are fixed
 
-        TableWrite write = table.newWrite();
-        TableCommit commit = table.newCommit("user");
+        TableWrite write = table.newWrite(commitUser);
+        TableCommit commit = table.newCommit(commitUser);
         write.write(GenericRowData.of(1));
-        commit.commit("0", write.prepareCommit(true));
+        commit.commit(0, write.prepareCommit(true, 0));
         write.write(GenericRowData.of((Object) null));
-        commit.commit("1", write.prepareCommit(true));
+        commit.commit(1, write.prepareCommit(true, 1));
         write.write(GenericRowData.of(2));
         write.write(GenericRowData.of(3));
         write.write(GenericRowData.of((Object) null));
-        commit.commit("2", write.prepareCommit(true));
+        commit.commit(2, write.prepareCommit(true, 2));
         write.write(GenericRowData.of(4));
         write.write(GenericRowData.of(5));
         write.write(GenericRowData.of(6));
-        commit.commit("3", write.prepareCommit(true));
+        commit.commit(3, write.prepareCommit(true, 3));
         write.close();
 
         hiveShell.execute(
@@ -868,22 +876,23 @@ public class TableStoreHiveStorageHandlerITCase {
                         Collections.emptyList(),
                         Collections.emptyList());
 
-        TableWrite write = table.newWrite();
-        TableCommit commit = table.newCommit("user");
+        TableWrite write = table.newWrite(commitUser);
+        TableCommit commit = table.newCommit(commitUser);
         write.write(
                 GenericRowData.of(
                         375, /* 1971-01-11 */
                         TimestampData.fromLocalDateTime(
-                                LocalDateTime.of(2022, 5, 17, 17, 29, 20))));
-        commit.commit("0", write.prepareCommit(true));
+                                LocalDateTime.of(2022, 5, 17, 17, 29, 20, 100_000_000))));
+        commit.commit(0, write.prepareCommit(true, 0));
         write.write(GenericRowData.of(null, null));
-        commit.commit("1", write.prepareCommit(true));
+        commit.commit(1, write.prepareCommit(true, 1));
         write.write(GenericRowData.of(376 /* 1971-01-12 */, null));
         write.write(
                 GenericRowData.of(
                         null,
-                        TimestampData.fromLocalDateTime(LocalDateTime.of(2022, 6, 18, 8, 30, 0))));
-        commit.commit("2", write.prepareCommit(true));
+                        TimestampData.fromLocalDateTime(
+                                LocalDateTime.of(2022, 6, 18, 8, 30, 0, 100_000_000))));
+        commit.commit(2, write.prepareCommit(true, 2));
         write.close();
 
         hiveShell.execute(
@@ -894,18 +903,18 @@ public class TableStoreHiveStorageHandlerITCase {
                                 "STORED BY '" + TableStoreHiveStorageHandler.class.getName() + "'",
                                 "LOCATION '" + path + "'")));
         Assert.assertEquals(
-                Collections.singletonList("1971-01-11\t2022-05-17 17:29:20.0"),
+                Collections.singletonList("1971-01-11\t2022-05-17 17:29:20.1"),
                 hiveShell.executeQuery("SELECT * FROM test_table WHERE dt = '1971-01-11'"));
         Assert.assertEquals(
-                Collections.singletonList("1971-01-11\t2022-05-17 17:29:20.0"),
+                Collections.singletonList("1971-01-11\t2022-05-17 17:29:20.1"),
                 hiveShell.executeQuery(
-                        "SELECT * FROM test_table WHERE ts = '2022-05-17 17:29:20'"));
+                        "SELECT * FROM test_table WHERE ts = '2022-05-17 17:29:20.1'"));
         Assert.assertEquals(
                 Collections.singletonList("1971-01-12\tNULL"),
                 hiveShell.executeQuery("SELECT * FROM test_table WHERE dt = '1971-01-12'"));
         Assert.assertEquals(
-                Collections.singletonList("NULL\t2022-06-18 08:30:00.0"),
+                Collections.singletonList("NULL\t2022-06-18 08:30:00.1"),
                 hiveShell.executeQuery(
-                        "SELECT * FROM test_table WHERE ts = '2022-06-18 08:30:00'"));
+                        "SELECT * FROM test_table WHERE ts = '2022-06-18 08:30:00.1'"));
     }
 }
