@@ -18,22 +18,16 @@
 
 package org.apache.flink.table.store.spark;
 
-import org.apache.flink.core.memory.DataInputViewStreamWrapper;
-import org.apache.flink.core.memory.DataOutputViewStreamWrapper;
 import org.apache.flink.table.store.table.source.Split;
 
 import org.apache.spark.sql.connector.read.InputPartition;
 
-import java.io.IOException;
-import java.io.ObjectInputStream;
-import java.io.ObjectOutputStream;
-
 /** A Spark {@link InputPartition} for table store. */
 public class SparkInputPartition implements InputPartition {
 
     private static final long serialVersionUID = 1L;
 
-    private transient Split split;
+    private final Split split;
 
     public SparkInputPartition(Split split) {
         this.split = split;
@@ -42,14 +36,4 @@ public class SparkInputPartition implements InputPartition {
     public Split split() {
         return split;
     }
-
-    private void writeObject(ObjectOutputStream out) throws IOException {
-        out.defaultWriteObject();
-        split.serialize(new DataOutputViewStreamWrapper(out));
-    }
-
-    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
-        in.defaultReadObject();
-        split = Split.deserialize(new DataInputViewStreamWrapper(in));
-    }
 }
